"""Module to solve Hermitian Matrix."""
# Reference: https://arxiv.org/abs/2102.11452
import numpy as np
from qiskit import aqua

from jarvis.core.kpoints import generate_kgrid
from qiskit import Aer

# from qiskit.circuit.library import EfficientSU2
import matplotlib.pyplot as plt
from jarvis.core.kpoints import Kpoints3D as Kpoints
from jarvis.db.figshare import get_hk_tb

plt.switch_backend("agg")


class HermitianSolver(object):
    """Solve a Hermitian matrix using quantum algorithms."""

    def __init__(self, mat=[]):
        """Initialize with a numpy Hermitian matrix."""
        N = int(np.ceil(np.log2(len(mat))))
        hk = np.zeros((2 ** N, 2 ** N), dtype="complex")
        hk[: mat.shape[0], : mat.shape[1]] = mat
        self.mat = hk
        if not self.check_hermitian():
            raise ValueError("Only implemented for Hermitian matrix.")

    def n_qubits(self):
        """Get number of qubits required."""
        return int(np.log2(len(self.mat)))

    def check_hermitian(self):
        """Check if a matrix is Hermitian."""
        adjoint = self.mat.conj().T
        return np.allclose(self.mat, adjoint)

    def run_vqe(
        self,
        backend=Aer.get_backend("statevector_simulator"),
        var_form=None,
        optimizer=None,
        reps=5,
        mode="min_val",
    ):
        """Run variational quantum eigensolver."""
        # N=int(np.ceil(np.log2(len(self.mat))))
        # hk = np.zeros((2**N,2**N),dtype='complex')
        # hk[:self.mat.shape[0], :self.mat.shape[1]] = self.mat
        N = self.n_qubits()
        if mode == "max_val":
            Hamil_mat = aqua.operators.MatrixOperator(-1 * self.mat)
            # Hamil_mat = MatrixOperator(-1 * self.mat)
        else:
            Hamil_mat = aqua.operators.MatrixOperator(self.mat)
            # Hamil_mat = MatrixOperator(self.mat)
        Hamil_qop = aqua.operators.op_converter.to_weighted_pauli_operator(
            Hamil_mat
        )
        if var_form is None:
            from qiskit.circuit.library import EfficientSU2

            var_form = EfficientSU2(N, reps=reps)
        if optimizer is None:
            vqe = aqua.algorithms.VQE(Hamil_qop, var_form)
            # vqe = VQE(Hamil_qop, var_form)
        else:
            vqe = aqua.algorithms.VQE(Hamil_qop, var_form, optimizer)
            # vqe = VQE(Hamil_qop, var_form, optimizer)
        vqe_result = vqe.run(backend)
        en = np.real(vqe_result["eigenvalue"])
        # params=vqe.optimal_params
        # circuit=vqe.construct_circuit(params)
        if mode == "max_val":
            en = -1 * en
        # states = np.sort(
        #    np.real(
        #        vqe.expectation.convert(
        #            StateFn(vqe.operator, is_measurement=True)
        #        ).to_matrix()
        #    )
        # )
        return en, vqe_result, vqe

    def run_numpy(self):
        """Obtain eigenvalues and vecs using Numpy solvers."""
        return np.linalg.eigh(self.mat)

    def run_qpe(self, n_ancillae=8):
        """Run quantum phase estimations."""
        quantum_instance = aqua.QuantumInstance(
            backend=Aer.get_backend("statevector_simulator"), shots=1
        )
        Hamil_mat = aqua.operators.MatrixOperator(self.mat)
        # Hamil_mat = MatrixOperator(self.mat)
        Hamil_qop = aqua.operators.op_converter.to_weighted_pauli_operator(
            Hamil_mat
        )
        # Hamil_qop = op_converter.to_weighted_pauli_operator(Hamil_mat)
        qpe = aqua.algorithms.QPE(Hamil_qop, num_ancillae=n_ancillae)
        qpe_result = qpe.run(quantum_instance)
        # qc = qpe.construct_circuit(measurement=True)
        print("qpe_result", qpe_result)
        return qpe_result["eigenvalue"], qpe_result, qpe

    def run_vqd(
        self,
        backend=Aer.get_backend("statevector_simulator"),
        var_form=None,
        optimizer=None,
        reps=5,
    ):
        """Run variational quantum deflation."""
        tmp = HermitianSolver(self.mat)
        max_eigval, vqe_result, vqe = tmp.run_vqe(
            backend=backend,
            var_form=var_form,
            optimizer=optimizer,
            reps=reps,
            mode="max_val",
        )
        eigvals = [max_eigval]
        eigstates = [vqe_result.eigenstate]
        for r in range(len(tmp.mat) - 1):
            val, vqe_result, vqe = tmp.run_vqe(
                backend=backend,
                var_form=var_form,
                optimizer=optimizer,
                reps=reps,
            )
            outer_prod = np.outer(
                vqe_result.eigenstate, np.conj(vqe_result.eigenstate).T
            )
            tmp.mat = tmp.mat - (val - max_eigval) * outer_prod
            eigvals.append(val)
            eigstates.append(vqe_result.eigenstate)
            tmp = HermitianSolver(tmp.mat)

        eigvals = np.array(eigvals)
        eigstates = np.array(eigstates)
        order = np.argsort(eigvals)
        eigvals = eigvals[order]
        eigstates = eigstates[order]
        return eigvals, eigstates


def get_bandstruct(
    w=[],
    atoms={},
    ef=0,
    line_density=1,
    ylabel="Energy (cm-1)",
    font=22,
    filename="bands.png",
    savefig=True,
    neigs=None,
    max_nk=None,
    tol=None,
):
    """Compare bandstructures using quantum algos."""
    info = {}
    kpoints = Kpoints().kpath(atoms, line_density=line_density)
    labels = kpoints.to_dict()["labels"]
    kpts = kpoints.to_dict()["kpoints"]
    print("kpts", len(kpts))

    eigvals_q = []
    eigvals_np = []
    for ii, i in enumerate(kpts):
        if max_nk is not None and ii == max_nk:
            break
            # For reducing CI/CD time
            print("breaking here", ii, max_nk)
        else:
            try:
                print("kp=", ii, i)
                hk = get_hk_tb(w=w, k=i)
                HS = HermitianSolver(hk)
                vqe_vals, _ = HS.run_vqd()
                np_vals, _ = HS.run_numpy()
                print("np_vals", np_vals)
                print("vqe_vals", vqe_vals)
                eigvals_q.append(vqe_vals)
                eigvals_np.append(np_vals)
                # break
                if (
                    neigs is not None
                    and isinstance(neigs, int)
                    and neigs == len(eigvals_q)
                ):
                    break
            except Exception as exp:
                print(exp)
                pass
    eigvals_q = 3.14 * np.array(eigvals_q)
    eigvals_np = 3.14 * np.array(eigvals_np)

    for ii, i in enumerate(eigvals_q.T - ef):
        if ii == 0:
            plt.plot(i, c="b", label="VQD")
        else:
            plt.plot(i, c="b")

    for ii, i in enumerate(eigvals_np.T - ef):
        if ii == 0:
            plt.plot(i, c="r", label="Numpy")
        else:
            plt.plot(i, c="r")
    new_kp = []
    new_labels = []
    count = 0
    kp = np.arange(len(kpts))
    for i, j in zip(kp, labels):
        if j != "":
            if count > 1 and count < len(labels) - 1:
                if labels[count] != labels[count + 1]:
                    new_kp.append(i)
                    new_labels.append("$" + str(j) + "$")
            else:
                new_kp.append(i)
                new_labels.append("$" + str(j) + "$")
        count += 1
    info["eigvals_q"] = list(eigvals_q.tolist())
    info["eigvals_np"] = list(eigvals_np.tolist())
    info["kpts"] = list(kpts)
    info["new_kp"] = list(np.array(new_kp).tolist())
    info["new_labels"] = list(new_labels)
    info["ef"] = ef
    print(info)
    if tol is not None:
        plt.ylim([tol, np.max(eigvals_q)])
    plt.rcParams.update({"font.size": font})
    plt.xticks(new_kp, new_labels)
    plt.ylabel(ylabel)
    plt.legend()
    plt.tight_layout()

    if savefig:
        plt.savefig(filename)
        plt.close()
    else:
        plt.show()
    return info


def get_dos(
    w=[],
    grid=[2, 1, 1],
    proj=None,
    efermi=0.0,
    xrange=None,
    nenergy=100,
    sig=0.02,
    use_dask=True,
    filename="dos.png",
    savefig=True,
):
    """Get density of states."""
    nwan = int(np.ceil(np.log2(w.nwan))) ** 2
    kpoints = generate_kgrid(grid=grid)
    nk = len(kpoints)
    q_vals = np.zeros((nk, nwan), dtype=float)
    np_vals = np.zeros((nk, nwan), dtype=float)
    pvals = np.zeros((nk, nwan), dtype=float)
    # if use_dask:
    # def get_vqd_vals(k):
    #    hk = get_hk_tb(w=w, k=k)
    #    HS = HermitianSolver(hk)
    #    vqe_vals, _ = HS.run_vqd()
    #    return vqe_vals

    # values=[delayed(get_vqd_vals)(k) for k in  kpoints]
    # resultsDask = compute(*values, scheduler='processes')
    for i, k in enumerate(kpoints):
        hk = get_hk_tb(w=w, k=k)
        HS = HermitianSolver(hk)
        vqe_vals, _ = HS.run_vqd()
        n_vals, _ = HS.run_numpy()
        q_vals[i, :] = vqe_vals
        np_vals[i, :] = n_vals
        print("np_vals", n_vals)
        print("vqe_vals", vqe_vals)

    if xrange is None:
        vmin = np.min(q_vals[:])
        vmax = np.max(q_vals[:])
        vmin2 = vmin - (vmax - vmin) * 0.05
        vmax2 = vmax + (vmax - vmin) * 0.05
        xrange = [vmin2, vmax2]
        # plt.xlim(xrange)

    energies = np.arange(
        xrange[0], xrange[1] + 1e-5, (xrange[1] - xrange[0]) / float(nenergy),
    )
    dos = np.zeros(np.size(energies))
    pdos = np.zeros(np.size(energies))

    v = q_vals

    #   condmin = np.min(v[v > 0.0])
    #   valmax = np.max(v[v < 0.0])
    #   print("DOS BAND GAP ", condmin - valmax, "    ", valmax, " ", condmin)

    c = -0.5 / sig ** 2
    for i in range(np.size(energies)):
        arg = c * (v - energies[i]) ** 2
        dos[i] = np.sum(np.exp(arg))
        if proj is not None:
            pdos[i] = np.sum(np.exp(arg) * pvals)

    de = energies[1] - energies[0]
    dos = dos / sig / (2.0 * np.pi) ** 0.5 / float(nk)
    if proj is not None:
        pdos = pdos / sig / (2.0 * np.pi) ** 0.5 / float(nk)
    print("np.sum(dos) ", np.sum(dos * de))
    if proj is not None:
        print("np.sum(pdos) ", np.sum(pdos * de))
    plt.plot(energies, dos)
    plt.savefig(filename)
    plt.close()
    return energies, dos, pdos


"""
if __name__ == "__main__":
    from jarvis.db.figshare import (
        get_wann_phonon,
        get_hk_tb,
        get_wann_electron,
    )
    from jarvis.core.atoms import Atoms
    from jarvis.db.jsonutils import dumpjson

    w, ef, atoms = get_wann_electron("JVASP-816")
    info = get_bandstruct(
        w=w,
        line_density=5,
        atoms=atoms,
        ef=ef,
        filename="Alelect.png",
        ylabel="Energy (eV)",
    )
    dumpjson(data=info, filename="Alelect.json")
    w, atoms = get_wann_phonon("JVASP-54", factor=34.3)
    info = get_bandstruct(
        w=w,
        line_density=11,
        atoms=atoms,
        tol=0.1,
        filename="Alphon.png",
        ylabel="Freq.(cm$^{-1}$)",
    )
    # dumpjson(data=info,filename='Alphon.json')
    hk = get_hk_tb(w=w, k=[0.0, 0.0, 0.0])
    H = HermitianSolver(hk)
    en, vqe_result, vqe = H.run_vqe(mode="max_val")
    print("en=", en)
    # eigs,vecs=H.run_vqd()
    # print(eigs)
    # print(vecs)

    # eigs, vecs = H.run_numpy()
    # print(eigs)
    # print(vecs)
    # get_bandstruct(w=w, atoms=atoms, tol=0.1)
    # get_dos(w=w)
    # H.run_qpe()
"""



"""Methods for handling input/output files for tequilla."""
# Reference: https://arxiv.org/abs/2102.11452
import numpy
import numpy as np
import tequila as tq
from jarvis.io.qiskit.inputs import HermitianSolver


def get_eigvals(mat=[]):
    """Get eigenvalues for Hermitianmatrix."""
    herm = HermitianSolver(mat)
    matrix = herm.mat
    H = tq.paulis.Zero()
    rows = matrix.shape[0]
    columns = matrix.shape[0]
    n_qubits = herm.n_qubits()
    print("true values", np.linalg.eigh(matrix)[0])
    for i in range(rows):
        for j in range(columns):
            H += matrix[i, j] * tq.paulis.KetBra(
                ket=i, bra=j, n_qubits=n_qubits
            )

    hermitian, anti = H.split()
    eigenValues, eigenVectors = numpy.linalg.eigh(hermitian.to_matrix())
    # print(tq.QubitWaveFunction(eigenVectors[:, 2]))

    circuits = []
    energies = []
    factor = 22
    # TODO: replace factor with reverse VQE value
    opt_variables = {}
    for i in range(rows):
        U = tq.gates.Ry(angle=(0, i), target=0)
        U += tq.gates.Ry(angle=(1, i), target=1)
        U += tq.gates.Ry(angle=(2, i), target=2)
        active_variables = U.extract_variables()
        E = tq.ExpectationValue(H=hermitian, U=U)
        ex_objective = E
        P1 = tq.paulis.Projector("1.0*|000>")
        for k in range(i):
            S = tq.ExpectationValue(H=P1, U=U + circuits[k].dagger())
            ex_objective += factor * abs(energies[k]) * S
        opt_variables = {
            **opt_variables,
            **{k: 1.0e-3 for k in active_variables},
        }
        result = tq.minimize(
            objective=ex_objective,
            method="bfgs",
            variables=active_variables,
            initial_values=opt_variables,
        )
        circuits.append(U)
        energies.append(result.energy)
        opt_variables = {**opt_variables, **result.variables}
    return energies


# def su2_circuit(n_qubits, layers, label):
#     """Get  SU2 circuit."""
#     U = tq.QCircuit()
#     for layer in range(layers):
#         for n in range(n_qubits):
#             U += tq.gates.Rz(target=n, angle=("z", n, layer, label))
#             U += tq.gates.Ry(target=n, angle=("y", n, layer, label))
#             If n < n_qubits-1:
#                 U += tq.gates.X(control=n, target=n+1)
#     return U

"""
if __name__ == "__main__":
    from jarvis.db.figshare import (
        get_wann_phonon,
        get_hk_tb,
        get_wann_electron,
    )

    w, atoms, o = get_wann_electron("JVASP-816")
    # w, atoms = get_wann_phonon("JVASP-816")
    hk = get_hk_tb(w=w, k=[0.0, 0.0, 0])
    print("hk", hk)
    H = HermitianSolver(hk)
    ens = get_eigvals(hk)
    print(ens)
"""



"""Module to get Wannier TB format for Force-constants."""
import numpy as np
from datetime import datetime


def get_phonon_hr(
    fcmat,
    smallest_vectors,
    mass,
    multi,
    super_pos,
    p2s_map,
    s2p_map,
    num_satom,
    num_patom,
    out_file,
):
    """
    Get phonon-TB Hamiltonian similar to wannier90_hr.dat.

    Its build by FORCE_CONSTANTS
    num_satom: number of atoms in super cell
    num_patom: number of atoms in primitive cell
    fcmat: force constansts num_satom*num_satom*3*3
    Forked from WannierTools package, please cite it
    along with jarvis-tools f you use this part of the code.
    """
    # maximum dimension for hr matrix
    ndim = 51
    sdim = 10
    nrpt_max = 51 ** 3
    # hr matrix
    norbs = num_patom * 3
    hr_mat = np.zeros((ndim, ndim, ndim, norbs, norbs), dtype=np.complex128)
    hr_mat0 = np.zeros((nrpt_max, norbs, norbs), dtype=np.complex128)
    # WS points
    rpts = np.zeros((nrpt_max, 3), dtype=np.int32)
    # degeneracy
    dege = np.zeros((nrpt_max), dtype=np.int32)

    for iatom in range(num_patom):  # atoms in primitive cell
        for jatom in range(num_patom):  # atoms in primitive cell
            mass_sqrt = np.sqrt(mass[iatom] * mass[jatom])
            for katom in range(num_satom):  # atoms in supercell
                if s2p_map[katom] != p2s_map[jatom]:
                    continue
                for rr in range(np.int32(multi[katom, iatom])):
                    # find which rpt
                    rvec = (
                        smallest_vectors[katom, iatom, rr]
                        + super_pos[p2s_map[iatom]]
                        - super_pos[s2p_map[katom]]
                    )
                    for ii in range(3):
                        if abs(rvec[ii]) < 1.0e-6:
                            rvec[ii] = 0.0
                    rx = np.int32(np.round(rvec[0]))
                    ry = np.int32(np.round(rvec[1]))
                    rz = np.int32(np.round(rvec[2]))
                    idx = iatom * 3
                    idy = jatom * 3
                    rx = rx + sdim
                    ry = ry + sdim
                    rz = rz + sdim
                    hr_mat[rx, ry, rz, idx : idx + 3, idy : idy + 3] = fcmat[
                        p2s_map[iatom], katom
                    ] / (multi[katom, iatom] * mass_sqrt)

    # collect all hr at rpts with none-zero data
    irpt = 0  # count for useful rpts
    for rx in range(-sdim, sdim + 1):
        ixo = rx + sdim
        for ry in range(-sdim, sdim + 1):
            iyo = ry + sdim
            for rz in range(-sdim, sdim + 1):
                izo = rz + sdim
                if (
                    abs(hr_mat[ixo, iyo, izo, :, :]).sum() < 1.0e-9
                ):  # ommit too small
                    continue
                dege[irpt] = 1
                rpts[irpt, 0] = rx
                rpts[irpt, 1] = ry
                rpts[irpt, 2] = rz
                hr_mat0[irpt, :, :] = hr_mat[ixo, iyo, izo, :, :]
                irpt = irpt + 1
    nrpt = irpt
    dege_rpts = dege[0:nrpt]
    norbs = num_patom * 3
    with open(out_file, "w") as f:
        line = (
            " Writen on "
            + str(datetime.now())
            + "\n"
            + "          "
            + str(norbs)
            + "\n"
            + "        "
            + str(nrpt)
            + "\n"
        )
        f.write(line)
        nl = np.int32(np.ceil(nrpt / 15.0))
        for n in range(nl):
            line = "    " + "    ".join(
                [str(np.int32(i)) for i in dege_rpts[n * 15 : (n + 1) * 15]]
            )
            f.write(line)
            f.write("\n")
        for irpt in range(nrpt):
            rx = rpts[irpt, 0]
            ry = rpts[irpt, 1]
            rz = rpts[irpt, 2]
            for jatomorb in range(norbs):
                for iatomorb in range(norbs):
                    rp = hr_mat0[irpt, iatomorb, jatomorb].real
                    ip = hr_mat0[irpt, iatomorb, jatomorb].imag
                    tmp = "{:8d}{:8d}{:8d}{:8d}{:8d}{:20.10f}{:20.10f}\n"
                    line = tmp.format(
                        rx, ry, rz, iatomorb + 1, jatomorb + 1, rp, ip
                    )
                    f.write(line)



from jarvis.analysis.interface.zur import (
    ZSLGenerator,
    get_hetero_type,
    make_interface,
    add_atoms,
)
from jarvis.core.atoms import Atoms
from jarvis.io.vasp.inputs import Poscar
import os
from jarvis.db.figshare import get_jid_data


def get_2d_hetero_jids(jid1="JVASP-664", jid2="JVASP-52"):
    from jarvis.db.figshare import get_jid_data

    m1 = get_jid_data(jid1)["atoms"]
    m2 = get_jid_data(jid2)["atoms"]
    mat1 = Atoms.from_dict(m1)
    mat2 = Atoms.from_dict(m2)
    vac = max(mat1.lattice_mat[2][2], mat2.lattice_mat[2][2])
    print("jid1", jid1)
    print(mat1)
    print("jid2", jid2)
    print(mat2)
    combined = make_interface(
        film=mat1.center_around_origin(),
        # max_area=1000,
        max_area=400,
        max_area_ratio_tol=0.09,
        # ltol=0.5,
        ltol=0.05,
        # atol=1.1,
        atol=0.1,
        subs=mat2.center_around_origin(),
    )["interface"]
    return combined


# Good 2D examples
jids = [
    "JVASP-664",
    "JVASP-667",
    "JVASP-661",
    "JVASP-688",
    "JVASP-652",
    "JVASP-676",
    "JVASP-6841",
    "JVASP-771",
    "JVASP-780",
    "JVASP-5899",
    "JVASP-646",
    "JVASP-5956",
    "JVASP-649",
    "JVASP-658",
    "JVASP-744",
    "JVASP-5872",
    "JVASP-76196",
    "JVASP-5983",
    "JVASP-20002",
    "JVASP-27836",
    "JVASP-27862",
    "JVASP-6838",
    "JVASP-687",
    "JVASP-6613",
    "JVASP-6667",
    "JVASP-6028",
    "JVASP-6079",
    "JVASP-5944",
    "JVASP-680",
    "JVASP-6955",
    "JVASP-6901",
    "JVASP-19987",
    "JVASP-6238",
    "JVASP-13600",
    "JVASP-27940",
    "JVASP-27775",
    "JVASP-27780",
    "JVASP-730",
    "JVASP-5929",
    "JVASP-19989",
    "JVASP-792",
    "JVASP-6268",
    "JVASP-6181",
    "JVASP-12027",
    "JVASP-27785",
    "JVASP-28152",
    "JVASP-28268",
    "JVASP-6994",
    "JVASP-31373",
    "JVASP-762",
    "JVASP-756",
    "JVASP-27724",
    "JVASP-13514",
    "JVASP-19987",
    "JVASP-31368",
    "JVASP-6007",
    "JVASP-741",
    "JVASP-720",
    "JVASP-789",
    "JVASP-777",
    "JVASP-5932",
    "JVASP-728",
    "JVASP-673",
    "JVASP-60477",
    "JVASP-6742",
    "JVASP-12064",
    "JVASP-6034",
    "JVASP-5935",
    "JVASP-60525",
    "JVASP-14431",
    "JVASP-13632",
    "JVASP-31379",
    "JVASP-28106",
    "JVASP-27864",
    "JVASP-27855",
    "JVASP-5950",
    "JVASP-696",
    "JVASP-783",
    "JVASP-13526",
    "JVASP-13541",
    "JVASP-31353",
    "JVASP-5926",
    "JVASP-5959",
    "JVASP-726",
    "JVASP-60244",
    "JVASP-13536",
    "JVASP-27940",
    "JVASP-60497",
    "JVASP-705",
    "JVASP-27978",
    "JVASP-27865",
    "JVASP-5977",
    "JVASP-5974",
    "JVASP-750",
    "JVASP-738",
    "JVASP-655",
    "JVASP-31356",
    "JVASP-6862",
    "JVASP-765",
    "JVASP-13586",
]


def test_zur():
    m1 = get_jid_data("JVASP-664")["atoms"]
    m2 = get_jid_data("JVASP-652")["atoms"]
    s1 = Atoms.from_dict(m1)
    s2 = Atoms.from_dict(m2)
    info = make_interface(film=s1, subs=s2)
    combined = info["interface"]
    assert (
        round(info["mismatch_u"], 3),
        round(info["mismatch_angle"], 3),
    ) == (-0.041, 0.0,)


def test_mos2_bn():
    intf = get_2d_hetero_jids(jid1="JVASP-688", jid2="JVASP-664")
    print(intf)
    print(intf.density)
    assert round(intf.density, 4) == round(1.7346542883452571, 4)


def test_2d_interface():
    jids = [
        "JVASP-649",
        "JVASP-652",
        "JVASP-658",
        "JVASP-664",
        "JVASP-688",
        "JVASP-6841",
        "JVASP-5983",
        "JVASP-60244",
        "JVASP-60389",
        "JVASP-76195",
    ]
    # jids = ["JVASP-652", "JVASP-664"]
    jids = ["JVASP-76195", "JVASP-5983"]
    jids = ["JVASP-76195", "JVASP-60389"]
    # jids = ["JVASP-76195", "JVASP-60244"]
    jids = ["JVASP-688", "JVASP-664"]
    jids = ["JVASP-652", "JVASP-646"]
    jids = ["JVASP-652", "JVASP-658"]  # WS2,WSe2
    count = 0
    for i in jids:
        for j in jids:
            if count < 100 and i != j:
                # try:
                intf = get_2d_hetero_jids(jid1=i, jid2=j)
                if intf.num_atoms < 200:

                    count = count + 1
                    print(i, j)
                    p = Poscar(intf)
                    filename = "POSCAR-" + str(i) + "_" + str(j) + ".vasp"
                    p.comment = "Surf@" + str(i) + "_" + str(j)
                    print(p)
                    # p.write_file(filename)
                    print()
                    print()
                    print()
            # except:
            #  pass


def test_type():
    B = {}
    B["scf_vbm"] = -5
    B["scf_cbm"] = -4
    B["avg_max"] = -2
    A = {}
    A["scf_vbm"] = -7
    A["scf_cbm"] = -6
    A["avg_max"] = -2
    int_type, stack = get_hetero_type(A=A, B=B)
    assert (int_type, stack) == ("III", "BA")
    print(int_type, stack)
    int_type, stack = get_hetero_type(A=B, B=A)
    print(int_type, stack)

    B = {}
    B["scf_vbm"] = -5
    B["scf_cbm"] = -4
    B["avg_max"] = -1
    A = {}
    A["scf_vbm"] = -7
    A["scf_cbm"] = -3
    A["avg_max"] = -1
    int_type, stack = get_hetero_type(A=B, B=A)
    print(int_type, stack)

    B = {}
    B["scf_vbm"] = -4
    B["scf_cbm"] = -2
    B["avg_max"] = -1
    A = {}
    A["scf_vbm"] = -7
    A["scf_cbm"] = -3
    A["avg_max"] = -1
    int_type, stack = get_hetero_type(A=B, B=A)
    print(int_type, stack)


jids = [
    "JVASP-649",
    "JVASP-652",
    "JVASP-658",
    "JVASP-664",
    "JVASP-688",
    "JVASP-6841",
    "JVASP-5983",
    "JVASP-60244",
    "JVASP-60389",
    "JVASP-76195",
]


def test_2d_hetero():
    count = 0
    for i in jids:
        for j in jids:
            if count < 10 and i != j:
                try:
                    intf = get_2d_hetero_jids(jid1=i, jid2=j)
                    ats = intf.get_string(cart=False)
                    if intf.num_atoms < 200:
                        count = count + 1
                        print(i, j)
                        print(ats)
                        print()
                        print()
                        print()
                except:
                    pass


def test_metal_ceramic_interface():
    m1 = get_jid_data(jid="JVASP-816", dataset="dft_3d")["atoms"]
    m2 = get_jid_data(jid="JVASP-32", dataset="dft_3d")["atoms"]
    mat_Al = Atoms.from_dict(m1)
    mat_Al2O3 = Atoms.from_dict(m2)
    from jarvis.analysis.defects.surface import Surface

    mat1 = Surface(atoms=mat_Al, indices=[1, 1, 1], layers=3).make_surface()
    mat2 = Surface(atoms=mat_Al2O3, indices=[0, 0, 1], layers=1).make_surface()
    combined = make_interface(
        film=mat1,  # .center_around_origin(),
        max_area=500,
        max_area_ratio_tol=0.09,
        ltol=0.01,
        apply_strain=True,
        subs=mat2,  # .center_around_origin(),
    )["interface"]
    print(combined)
    # from ase.lattice.surface import surface
    # from pymatgen.io.ase import AseAtomsAdaptor
    # from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
    # from pymatgen.io.vasp.inputs import Poscar
    # mat_cvn = SpacegroupAnalyzer(mat_Al.pymatgen_converter()).get_conventional_standard_structure()
    # ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    # ase_slab = surface(ase_atoms, [1,1,1], 3)
    # ase_slab.center(vacuum=18, axis=2)
    # slab_pymatgen = AseAtomsAdaptor().get_structure(ase_slab)
    # slab_pymatgen.sort()
    # print (Poscar(slab_pymatgen))
    # print ()
    # print ()
    print(mat1.center_around_origin().get_string(cart=False))


# test_mos2_bn()
# test_2d_interface()
# test_metal_ceramic_interface()



import os
from jarvis.analysis.thermodynamics.energetics import form_enp

from jarvis.io.vasp.inputs import Poscar

p = os.path.join(os.path.dirname(__file__), "POSCAR")


def test_form_en():
    atoms = Poscar.from_file(p).atoms

    total_energy = -9.974648
    fen = form_enp(atoms=atoms, total_energy=total_energy)
    assert fen == -0.40429
    # print ('fen', fen)


# test_form_en()



from jarvis.core.composition import Composition


def test_comp():
    comp = {"Li": 2, "O": 4}
    cc = Composition(comp)

    assert (
        cc.prototype,
        cc.formula,
        cc.reduced_formula,
        round(cc.weight, 4),
        cc.to_dict(),
    ) == ("AB2", "Li2O4", "LiO2", 77.8796, comp)
    c = Composition.from_string("Al2O3Al5Co6O1")
    td = c.to_dict()
    fd = Composition.from_dict(td)
    assert c.formula == "Al7Co6O4"
    arr = c.atomic_fraction_array



from jarvis.db.figshare import (
    data,
    get_ff_eneleast,
    get_wann_electron,
    get_wann_phonon,
    get_hk_tb,
    get_stm_2d_dataset,
)
from jarvis.db.webpages import Webpage


def test_figshare_download():

    data_2d = data(dataset="dft_2d")
    # data_3d = data(dataset="dft_3d")
    # data_ml = data(dataset="cfid_3d")
    data_ff = get_ff_eneleast()
    print("2d", len(data_2d))
    # print("3d", len(data_3d))
    # print("cfid3d", len(data_ml))
    print("ff", len(data_ff))
    # assert (len(data_2d), len(data_3d), len(data_ml), len(data_ff)) == (
    #    1070,
    #    36099,
    #    35984,
    #    3291,
    # )
    w, ef, atoms = get_wann_electron()
    hk = get_hk_tb(w=w)
    w, atoms = get_wann_phonon()
    w = Webpage(jid="JVASP-1002")
    info_mbj = w.get_dft_mbj_dielectric_function()
    info_opt = w.get_dft_semilocal_dielectric_function()
    info_pdos = w.get_dft_phonon_dos()
    info_edos = w.get_dft_electron_dos()
    k = w.list_keys()
    # dat = data(dataset="megnet")
    # dat = data(dataset="mp_3d")
    # dat = data(dataset="mp_3d_2020")
    # dat = data(dataset="qm9")
    # dat = data(dataset="aflow1")
    # dat = data(dataset="aflow2")
    # dat = data(dataset="oqmd_3d")
    # dat = data(dataset="twod_matpd")
    # dat = data(dataset="oqmd_3d_no_cfid")
    p, n = get_stm_2d_dataset()


# test_figshare_download()



"""This module provides classes to specify atomic structure."""
# import line_profiler
import numpy as np
import matplotlib.pyplot as plt
import math
from toolz.curried import pipe


def check_array(arr, type_=None, shape=None):
    """
    Check type and shape of np.ndarray.

    Follwing arguments are needed.
    Args:
      x: array to check

      type_: the type of the array (int, float), None to not check

      shape: the shape of the array, None to not check, -1 to ignore a
        dimension

    """
    if not isinstance(arr, np.ndarray):
        raise TypeError("not an array", arr)


def special_arange(dims):
    """
    Multiple dimensional arange.

    This could be extended to any dimension.
    Args:

      dims: sequence of ints of length 3

    Returns:
      an array of shape (np.prod(dims), 3) with each index having a
      different arrangement of arange.
    This function implements the equivalent of a multidimensional for
    loop, which feels like multidimensional arange (see the test)

    >>> dim0, dim1, dim2 = 2, 4, 3
    >>> arr_test = np.zeros((dim0 * dim1 * dim2, 3), dtype=int)
    >>> counter = 0
    >>> for i in np.arange(dim0):
    ...     for j in np.arange(dim1):
    ...         for k in np.arange(dim2):
    ...             arr_test[counter, 0] = i
    ...             arr_test[counter, 1] = j
    ...             arr_test[counter, 2] = k
    ...             counter += 1

    >>> arr_actual = special_arange((dim0, dim1, dim2))
    >>> print(arr_actual)
    [[0 0 0]
     [0 0 1]
     [0 0 2]
     [0 1 0]
     [0 1 1]
     [0 1 2]
     [0 2 0]
     [0 2 1]
     [0 2 2]
     [0 3 0]
     [0 3 1]
     [0 3 2]
     [1 0 0]
     [1 0 1]
     [1 0 2]
     [1 1 0]
     [1 1 1]
     [1 1 2]
     [1 2 0]
     [1 2 1]
     [1 2 2]
     [1 3 0]
     [1 3 1]
     [1 3 2]]
    >>> assert np.all(arr_actual == arr_test)

    """
    # This can be made much more functional and for arbitrary
    # dimensions if necessary
    dim0, dim1, dim2 = dims
    i = np.repeat(np.repeat(np.arange(dim0), dim2), dim1)
    j = np.tile(np.repeat(np.arange(dim1), dim2), dim0)
    k = np.tile(np.tile(np.arange(dim2), dim1), dim0)
    return np.concatenate((i[:, None], j[:, None], k[:, None]), axis=-1)


def calc_structure_data(coords, box, all_symbs, c_size):
    """
    Calcuate a dictionary of structure data.

    Args:

      coords: the coordinates for each element

      box: the lattic matrix

      all_symbs: the elements

      c_size: the c size

    Returns:
      a set of structure data

    >>> coords = np.array([[0, 0, 0], [0.25, 0.2, 0.25]])
    >>> lat = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    >>> box = np.array(lat)
    >>> elements = ["Si", "Si"]
    >>> c_size = 10.0
    >>> data = calc_structure_data(coords, box, elements, c_size)
    >>> assert len(data['coords']) == 128
    >>> assert np.allclose(data['coords'][9], [0.    , 0.5   , 0.25  ])
    >>> assert np.all(data['dim'] == [4, 4, 4])
    >>> assert len(data['new_symbs']) == 128

    """
    check_array(coords, float, (len(all_symbs), 3))
    check_array(box, float, (3, 3))

    def make_coords(dim):
        return pipe(
            dim,
            special_arange,
            lambda x: (coords[:, None, :] + x[None, :, :])
            / dim[None, None, :],
            lambda x: np.reshape(x, (-1, 3)),
            lambda x: dict(
                coords=x,
                dim=dim,
                nat=len(x),
                lat=dim[:, None] * box,
                new_symbs=np.repeat(all_symbs, np.prod(dim)),
            ),
        )

    return make_coords((c_size / np.max(np.abs(box), axis=1)).astype(int) + 1)


class NeighborsAnalysis(object):
    """Get neighbor informations (RDF,ADF,DDF) for Atoms object."""

    def __init__(
        self,
        atoms=None,
        max_n=500,
        rcut1=None,
        max_cut=10.0,
        rcut2=None,
        verbose=False,
    ):
        """
        Initialize the function.

        >>> box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
        >>> coords = [[0, 0, 0], [0.25, 0.2, 0.25]]
        >>> elements = ["Si", "Si"]
        >>> Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
        >>> distributions = NeighborsAnalysis(Si).get_all_distributions
        >>> distributions['rdf']
        1
        """
        self._atoms = atoms
        self.max_n = max_n
        self.max_cut = max_cut
        self.nb_warn = ""
        if rcut1 is None or rcut2 is None:
            rcut1, rcut2 = self.get_dist_cutoffs()
        self.rcut1 = rcut1
        self.rcut2 = rcut2
        self.verbose = verbose
        if self.nb_warn != "" and self.verbose:
            print(self.nb_warn)
            print(
                "Try setting higher max_n in the NeighborsAnalysis module"
                + atoms.get_string()
            )

    def get_structure_data(self, c_size=10.0):
        """Provide non-repetitive structure information."""
        return calc_structure_data(
            self._atoms.frac_coords,
            self._atoms.lattice_mat,
            self._atoms.elements,
            c_size,
        )

    # @profile
    # kernprof -v -l neighbors.py
    def nbor_list(self, rcut=10.0, c_size=12.0):
        """Generate neighbor info."""
        max_n = self.max_n
        nbor_info = {}
        struct_info = self.get_structure_data(c_size)
        coords = np.array(struct_info["coords"])
        nat = struct_info["nat"]
        new_symbs = struct_info["new_symbs"]
        lat = np.array(struct_info["lat"])
        different_bond = {}
        nn = np.zeros((nat), dtype="int")
        # print ('max_n, nat',max_n, nat)
        dist = np.zeros((max_n, nat))
        nn_id = np.zeros((max_n, nat), dtype="int")
        bondx = np.zeros((max_n, nat))
        bondy = np.zeros((max_n, nat))
        bondz = np.zeros((max_n, nat))
        for i in range(nat):
            for j in range(i + 1, nat):
                diff = coords[i] - coords[j]
                ind = np.where(np.fabs(diff) > np.array([0.5, 0.5, 0.5]))
                diff[ind] -= np.sign(diff[ind])
                new_diff = np.dot(diff, lat)
                dd = np.linalg.norm(new_diff)
                if dd < rcut and dd >= 0.1:
                    sumb_i = new_symbs[i]
                    sumb_j = new_symbs[j]
                    comb = "_".join(
                        sorted(str(sumb_i + "_" + sumb_j).split("_"))
                    )
                    different_bond.setdefault(comb, []).append(dd)

                    # print ('dd',dd)
                    nn_index = nn[i]  # index of the neighbor
                    nn_index1 = nn[j]  # index of the neighbor
                    if nn_index < max_n and nn_index1 < max_n:
                        nn[i] = nn[i] + 1
                        dist[nn_index][i] = dd  # nn_index counter id
                        nn_id[nn_index][i] = j  # exact id
                        bondx[nn_index][i] = new_diff[0]
                        bondy[nn_index][i] = new_diff[1]
                        bondz[nn_index][i] = new_diff[2]
                        nn[j] = nn[j] + 1
                        dist[nn_index1][j] = dd  # nn_index counter id
                        nn_id[nn_index1][j] = i  # exact id
                        bondx[nn_index1][j] = -new_diff[0]
                        bondy[nn_index1][j] = -new_diff[1]
                        bondz[nn_index1][j] = -new_diff[2]
                    else:
                        self.nb_warn = (
                            "Very large nearest neighbors observed "
                            + str(nn_index)
                        )
        nbor_info["dist"] = dist
        nbor_info["nat"] = nat
        nbor_info["nn_id"] = nn_id
        nbor_info["nn"] = nn
        nbor_info["bondx"] = bondx
        nbor_info["bondy"] = bondy
        nbor_info["bondz"] = bondz
        nbor_info["different_bond"] = different_bond
        # print ('nat',nat)

        return nbor_info

    def get_rdf(self, plot=False):
        """Calculate radial distribution function."""
        nbor_info = self.nbor_list(c_size=2 * self.max_cut + 1)
        # nbor_info = self.nbor_list(c_size=21.0)
        n_zero_d = nbor_info["dist"][np.nonzero(nbor_info["dist"])]
        hist, bins = np.histogram(
            n_zero_d.ravel(), bins=np.arange(0.1, 10.2, 0.1)
        )
        const = float(nbor_info["nat"]) / float(self._atoms.num_atoms)
        hist = hist / float(const)
        shell_vol = (
            4.0
            / 3.0
            * np.pi
            * (np.power(bins[1:], 3) - np.power(bins[:-1], 3))
        )
        number_density = self._atoms.num_atoms / self._atoms.volume
        rdf = (
            hist / shell_vol / number_density / self._atoms.num_atoms
        )  # /len(n_zero_d)
        nn = rdf / self._atoms.num_atoms
        if plot:
            plt.plot(bins[:-1], rdf)
            plt.savefig("rdf.png")
            plt.close()
        return bins[:-1], rdf, nn

    def get_dist_cutoffs(self):
        """
        Get different distance cut-offs.

        Args:

            s: Structure object

        Returns:
               rcut: max-cutoff to ensure all the element-combinations
               are included, used in calculating angluar distribution
               upto first neighbor

               rcut1: decide first cut-off based on total RDF and a buffer
               (previously used in dihedrals, but not used now in the code)

               rcut2: second neighbor cut-off

               rcut_dihed: specialized cut-off for dihedrals to avaoid large
               bonds such as N-N, uses average bond-distance and standard
               deviations
        """
        max_cut = self.max_cut
        x, y, z = self.get_rdf()
        arr = []
        for i, j in zip(x, z):
            if j > 0.0:
                arr.append(i)
        rcut_buffer = 0.11
        io1 = 0
        io2 = 1
        io3 = 2
        try:
            delta = arr[io2] - arr[io1]
            while delta < rcut_buffer and arr[io2] < max_cut:
                io1 = io1 + 1
                io2 = io2 + 1
                io3 = io3 + 1
                delta = arr[io2] - arr[io1]
            rcut1 = (arr[io2] + arr[io1]) / float(2.0)
        except Exception:
            print("Warning:Setting first nbr cut-off as minimum bond-angle")
            rcut1 = arr[0]
            pass
        try:
            delta = arr[io3] - arr[io2]
            while (
                delta < rcut_buffer
                and arr[io3] < max_cut
                and arr[io2] < max_cut
            ):
                io2 = io2 + 1
                io3 = io3 + 1
                delta = arr[io3] - arr[io2]
            rcut2 = float(arr[io3] + arr[io2]) / float(2.0)
        except Exception:
            print("Warning:Setting first and second nbr cut-off equal")
            print("You might consider increasing max_n parameter")
            rcut2 = rcut1
            pass
        # rcut, rcut_dihed = get_prdf(s=s)
        # rcut_dihed=min(rcut_dihed,max_dihed)
        return rcut1, rcut2

    def ang_dist(self, nbor_info={}, plot=False):
        """
        Get  angular distribution function upto first neighbor.

        Args:
            struct_info: struct information

            max_n: maximum number of neigbors

            c_size: max. cell size

            plot: whether to plot distributions


        Retruns:

            ang_hist1: Angular distribution upto first cut-off

            ang_bins1: angle bins
        """
        # rcut1,rcut2=self.get_dist_cutoffs()
        # rcut=rcut1
        # nbor_info=self.nbor_list()
        znm = 0
        nat = nbor_info["nat"]
        dist = nbor_info["dist"]
        # nn_id = nbor_info["nn_id"]
        nn = nbor_info["nn"]
        bondx = nbor_info["bondx"]
        bondy = nbor_info["bondy"]
        bondz = nbor_info["bondz"]

        ang_at = {}

        for i in range(nat):
            for in1 in range(nn[i]):
                # j1 = nn_id[in1][i]
                for in2 in range(in1 + 1, nn[i]):
                    # j2 = nn_id[in2][i]
                    nm = dist[in1][i] * dist[in2][i]
                    if nm != 0:
                        rrx = bondx[in1][i] * bondx[in2][i]
                        rry = bondy[in1][i] * bondy[in2][i]
                        rrz = bondz[in1][i] * bondz[in2][i]
                        cos = float(rrx + rry + rrz) / float(nm)
                        if cos <= -1.0:
                            cos = cos + 0.000001
                        if cos >= 1.0:
                            cos = cos - 0.000001
                        deg = math.degrees(math.acos(cos))
                        ang_at.setdefault(round(deg, 3), []).append(i)
                    else:
                        znm = znm + 1
        angs = np.array([float(i) for i in ang_at.keys()])
        norm = np.array(
            [float(len(i)) / float(len(set(i))) for i in ang_at.values()]
        )
        ang_hist, ang_bins = np.histogram(
            angs, weights=norm, bins=np.arange(1, 181.0, 1), density=False,
        )
        # if plot == True:
        #    plt.bar(ang_bins[:-1], ang_hist)
        #    plt.savefig("ang1.png")
        #    plt.close()

        return ang_hist, ang_bins

    def ang_dist_first(self, plot=False):
        """Get angular distribution upto first neighbor."""
        rcut1 = self.rcut1
        # rcut2 = self.rcut2
        nbor_info = self.nbor_list(rcut=rcut1)
        ang_hist, ang_bins = self.ang_dist(nbor_info=nbor_info)
        # print ('anghist',ang_hist)
        if plot:

            plt.plot(ang_bins[:-1], ang_hist)
            plt.savefig("adf1.png")
            plt.close()
        return ang_hist, ang_bins

    def ang_dist_second(self, plot=False):
        """Get angular distribution upto second neighbor."""
        # rcut1, rcut2 = self.get_dist_cutoffs()
        # rcut1 = self.rcut1
        rcut2 = self.rcut2
        # print ('rcut1,rcut2',rcut1,rcut2)
        nbor_info = self.nbor_list(rcut=rcut2)
        ang_hist, ang_bins = self.ang_dist(nbor_info=nbor_info)
        if plot:
            plt.plot(ang_bins[:-1], ang_hist)
            plt.savefig("adf2.png")
            plt.close()
        return ang_hist, ang_bins

    def get_ddf(self, plot=False):
        """Get dihedral distribution upto first neighbor."""
        # rcut1, rcut2 = self.get_dist_cutoffs()
        rcut1 = self.rcut1
        # rcut2 = self.rcut2
        nbor_info = self.nbor_list(rcut=rcut1)
        nat = nbor_info["nat"]
        # dist = nbor_info["dist"]
        nn_id = nbor_info["nn_id"]
        nn = nbor_info["nn"]
        bondx = nbor_info["bondx"]
        bondy = nbor_info["bondy"]
        bondz = nbor_info["bondz"]
        dih_at = {}
        for i in range(nat):
            for in1 in range(nn[i]):
                j1 = nn_id[in1][i]
                if j1 > i:
                    for in2 in range(nn[i]):
                        # all other nn of i that are not j
                        j2 = nn_id[in2][i]
                        if j2 != j1:
                            for in3 in range(
                                nn[j1]
                            ):  # all other nn of j that are not i
                                j3 = nn_id[in3][j1]
                                if j3 != i:
                                    v1 = []
                                    v2 = []
                                    v3 = []
                                    v1.append(bondx[in2][i])
                                    v1.append(bondy[in2][i])
                                    v1.append(bondz[in2][i])
                                    v2.append(-bondx[in1][i])
                                    v2.append(-bondy[in1][i])
                                    v2.append(-bondz[in1][i])
                                    v3.append(-bondx[in3][j1])
                                    v3.append(-bondy[in3][j1])
                                    v3.append(-bondz[in3][j1])
                                    v23 = np.cross(v2, v3)
                                    v12 = np.cross(v1, v2)
                                    theta = math.degrees(
                                        math.atan2(
                                            np.linalg.norm(v2)
                                            * np.dot(v1, v23),
                                            np.dot(v12, v23),
                                        )
                                    )
                                    if theta < 0.00001:
                                        theta = -theta
                                    # print "theta=",theta
                                    dih_at.setdefault(
                                        round(theta, 3), []
                                    ).append(i)
        dih = np.array([float(i) for i in dih_at.keys()])
        # dih1 = set(dih)
        # print "dih",dih1
        norm = np.array(
            [float(len(i)) / float(len(set(i))) for i in dih_at.values()]
        )

        dih_hist1, dih_bins1 = np.histogram(
            dih, weights=norm, bins=np.arange(1, 181.0, 1), density=False,
        )
        if plot:
            plt.plot(dih_bins1[:-1], dih_hist1)
            plt.savefig("dihedrals.png")
            plt.close()
        return dih_hist1, dih_bins1

    @property
    def get_all_distributions(self):
        """Get all distributions."""
        distributions = {}
        _, rdf, nn = self.get_rdf()
        ddf, _ = self.get_ddf()
        adfa, _ = self.ang_dist_first()
        adfb, _ = self.ang_dist_second()
        distributions["rdf"] = rdf
        distributions["ddf"] = ddf
        distributions["adfa"] = adfa
        distributions["adfb"] = adfb
        distributions["nn"] = nn
        return distributions

    def atomwise_radial_dist(self, rcut=10.0, c_size=0):
        """Get pair/radial distribution for each atom."""
        if rcut < c_size:
            rcut = c_size + 1
        nbor_info = self.nbor_list(rcut=rcut, c_size=c_size)
        nat = nbor_info["nat"]
        dist = nbor_info["dist"]
        atom_rdfs = []

        for i in range(nat):
            hist, bins = np.histogram(
                dist[:, i], bins=np.arange(0.1, rcut + 0.2, 0.1)
            )
            atom_rdfs.append(hist.tolist())
            if self.verbose:
                exact_dists = np.arange(0.1, rcut + 0.2, 0.1)[hist.nonzero()]
                print("exact_dists", exact_dists)
        return np.array(atom_rdfs)

    def atomwise_angle_dist(self, rcut=None, nbins=180, c_size=0):
        """Get angle distribution for each atom."""

        def angle(
            dist1, dist2, bondx1, bondx2, bondy1, bondy2, bondz1, bondz2,
        ):
            """Get an angle."""
            nm = dist1 * dist2
            rrx = bondx1 * bondx2
            rry = bondy1 * bondy2
            rrz = bondz1 * bondz2
            cos = (rrx + rry + rrz) / (nm)
            if cos <= -1.0:
                cos = cos + 0.000001
            if cos >= 1.0:
                cos = cos - 0.000001
            deg = math.degrees(math.acos(cos))
            return deg

        if rcut is None:
            rcut = self.rcut1
        nbor_info = self.nbor_list(rcut=rcut, c_size=c_size)
        atom_angles = []
        for i in range(nbor_info["nat"]):
            angles = [
                angle(
                    nbor_info["dist"][in1][i],
                    nbor_info["dist"][in2][i],
                    nbor_info["bondx"][in1][i],
                    nbor_info["bondx"][in2][i],
                    nbor_info["bondy"][in1][i],
                    nbor_info["bondy"][in2][i],
                    nbor_info["bondz"][in1][i],
                    nbor_info["bondz"][in2][i],
                )
                for in1 in range(nbor_info["nn"][i])
                for in2 in range(nbor_info["nn"][i])
                if in2 > in1
                and nbor_info["dist"][in1][i] * nbor_info["dist"][in2][i] != 0
            ]
            ang_hist, ang_bins = np.histogram(
                angles, bins=np.arange(1, nbins + 2, 1), density=False,
            )
            atom_angles.append(ang_hist)
            # print("fff",i,ang_hist)
            if self.verbose:
                exact_angles = np.arange(1, nbins + 2, 1)[ang_hist.nonzero()]
                print("exact_angles", exact_angles)
        # return (atom_angles)#/nbor_info['nat']
        return np.array(atom_angles)  # /nbor_info['nat']


"""
if __name__ == "__main__":
    from jarvis.core.atoms import Atoms

    box = [[5.493642, 0, 0], [0, 5.493642, 0], [0, 0, 5.493642]]
    elements = ["Si", "Si", "Si", "Si", "Si", "Si", "Si", "Si"]
    coords = [
        [0, 0, 0],
        [0.25, 0.25, 0.25],
        [0.000000, 0.500000, 0.500000],
        [0.250000, 0.750000, 0.750000],
        [0.500000, 0.000000, 0.500000],
        [0.750000, 0.250000, 0.750000],
        [0.500000, 0.500000, 0.000000],
        [0.750000, 0.750000, 0.250000],
    ]
    # box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    # coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    # elements = ["Si", "Si"]
    Si = Atoms(
        lattice_mat=box, coords=coords, elements=elements
    )  # .get_primitive_atoms
    ver = False  # True
    a = NeighborsAnalysis(Si, verbose=ver, max_cut=5).atomwise_radial_dist(
        c_size=5
    )
    a = NeighborsAnalysis(Si)
    print (a.nbor_list()['different_bond'][0])

    print(a[1], len(a), a[1].nonzero())
    a = NeighborsAnalysis(Si, verbose=ver, max_cut=5).atomwise_radial_dist(
        c_size=10
    )
    print(a[1], len(a), a[1].nonzero())
    a = NeighborsAnalysis(Si, verbose=ver, max_cut=5).atomwise_radial_dist(
        c_size=20
    )
    print(a[1], len(a), a[1].nonzero())
"""



"""Module to parse WIEN2k outputs."""
import numpy as np
import matplotlib.pyplot as plt

plt.switch_backend("agg")


def read_band_energy(
    energy_file="FeSe.energy", plot=True, band_plot="band.png"
):
    """Read case.energy file."""
    f = open(energy_file, "r")
    lines = f.read().splitlines()
    f.close()
    eigs = []
    eig_bands = []
    start = True
    for i in lines:
        sp = i.split()
        if len(sp) == 2 and start:
            tmp = float(sp[1])
            eig_bands.append(tmp)
        if len(sp) == 7:
            start = True
            if eig_bands != []:
                eigs.append(np.array(eig_bands))
            eig_bands = []
    eigs = np.array(eigs)
    if plot:
        for ii, i in enumerate(eigs):
            plt.plot([ii for j in range(len(i))], i, ".", c="b")
        plt.savefig(band_plot)
        plt.close()
    return eigs


def read_scf(scf_file="FeSe.scf"):
    """Read case.scf file."""
    f = open(scf_file, "r")
    lines = f.read().splitlines()
    f.close()
    info = {}
    efermies = []
    nelects = []
    nkpts = []
    totens = []
    for i in lines:
        if ":FER  : F E R M I - ENERGY(TETRAH.M.)" in i:
            efermies.append(
                float(i.split(":FER  : F E R M I - ENERGY(TETRAH.M.)=")[1])
            )
        if ":ENE  : ********** TOTAL ENERGY IN Ry" in i:
            totens.append(
                float(i.split(":ENE  : ********** TOTAL ENERGY IN Ry =")[1])
            )
        if ":NOE  : NUMBER OF ELECTRONS          =" in i:
            nelects.append(
                float(i.split(":NOE  : NUMBER OF ELECTRONS          =")[1])
            )
        if ":KPT   :      NUMBER OF K-POINTS:" in i:
            nkpts.append(
                float(i.split(":KPT   :      NUMBER OF K-POINTS:")[1])
            )
    info["efermi"] = efermies[-1]
    info["nelect"] = nelects[-1]
    info["nkpt"] = nkpts[-1]
    info["tot_en"] = totens[-1]
    return info


def band_eigvals(energy_file="FeSe.energy", plot=False, band_file="band.png"):
    """Get bandstructure eigenvalues."""
    f = open(energy_file, "r")
    lines = f.read().splitlines()
    f.close()
    eigs = []
    eig_bands = []
    start = True
    for i in lines:
        sp = i.split()
        if len(sp) == 2 and start:
            tmp = float(sp[1])
            eig_bands.append(tmp)
        if len(sp) == 7:
            start = True
            if eig_bands != []:
                eigs.append(eig_bands)
            eig_bands = []
    eigs = np.array(eigs)
    if plot:
        import matplotlib.pyplot as plt

        for i in eigs.T:
            plt.plot(i)
        plt.savefig(band_file)
        plt.close()
    return eigs


def read_spaghetti_ene(
    filename="ICSD-76748.spaghetti_ene",
):
    """Obtain data for plotting bandstructure."""
    f = open(filename, "r")
    lines = f.read().splitlines()
    f.close()
    k = []
    energy = []
    for i, ii in enumerate(lines):
        if "bandindex" not in ii:
            tmp = [float(j) for j in ii.split()]
            energy.append(tmp[-1])
            k.append(tmp[-2])
    k = np.array(k)
    energy = np.array(energy)
    return k, energy


"""
read_band_energy()
x = read_scf()
print(x)
"""



"""
Set of ranges for hyperparameters.

# Modified from  https://github.com/EpistasisLab/tpot
"""

import numpy as np


def classification_regression_params():
    """Set of hyperparameters."""
    regressor_config_dict = {
        "sklearn.linear_model.ElasticNetCV": {
            "l1_ratio": np.arange(0.0, 1.01, 0.05),
            "tol": [1e-5, 1e-4, 1e-3, 1e-2, 1e-1],
        },
        "sklearn.ensemble.ExtraTreesRegressor": {
            "n_estimators": [100],
            "max_features": np.arange(0.05, 1.01, 0.05),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
            "bootstrap": [True, False],
        },
        "sklearn.ensemble.GradientBoostingRegressor": {
            "n_estimators": [100],
            "loss": ["ls", "lad", "huber", "quantile"],
            "learning_rate": [1e-3, 1e-2, 1e-1, 0.5, 1.0],
            "max_depth": range(1, 11),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
            "subsample": np.arange(0.05, 1.01, 0.05),
            "max_features": np.arange(0.05, 1.01, 0.05),
            "alpha": [0.75, 0.8, 0.85, 0.9, 0.95, 0.99],
        },
        "sklearn.ensemble.AdaBoostRegressor": {
            "n_estimators": [100],
            "learning_rate": [1e-3, 1e-2, 1e-1, 0.5, 1.0],
            "loss": ["linear", "square", "exponential"],
        },
        "sklearn.tree.DecisionTreeRegressor": {
            "max_depth": range(1, 11),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
        },
        "sklearn.neighbors.KNeighborsRegressor": {
            "n_neighbors": range(1, 101),
            "weights": ["uniform", "distance"],
            "p": [1, 2],
        },
        "sklearn.linear_model.LassoLarsCV": {"normalize": [True, False]},
        "sklearn.svm.LinearSVR": {
            "loss": ["epsilon_insensitive", "squared_epsilon_insensitive"],
            "dual": [True, False],
            "tol": [1e-5, 1e-4, 1e-3, 1e-2, 1e-1],
            "C": [1e-4, 1e-3, 1e-2, 1e-1, 0.5, 1.0,
                  5.0, 10.0, 15.0, 20.0, 25.0],
            "epsilon": [1e-4, 1e-3, 1e-2, 1e-1, 1.0],
        },
        "sklearn.ensemble.RandomForestRegressor": {
            "n_estimators": [100],
            "max_features": np.arange(0.05, 1.01, 0.05),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
            "bootstrap": [True, False],
        },
        "sklearn.linear_model.RidgeCV": {},
        "xgboost.XGBRegressor": {
            "n_estimators": [100],
            "max_depth": range(1, 11),
            "learning_rate": [1e-3, 1e-2, 1e-1, 0.5, 1.0],
            "subsample": np.arange(0.05, 1.01, 0.05),
            "min_child_weight": range(1, 21),
            "nthread": [1],
            "objective": ["reg:squarederror"],
        },
        # Preprocesssors
        "sklearn.preprocessing.Binarizer":
        {"threshold": np.arange(0.0, 1.01, 0.05)},
        "sklearn.decomposition.FastICA": {"tol": np.arange(0.0, 1.01, 0.05)},
        "sklearn.cluster.FeatureAgglomeration": {
            "linkage": ["ward", "complete", "average"],
            "affinity": ["euclidean", "l1", "l2", "manhattan", "cosine"],
        },
        "sklearn.preprocessing.MaxAbsScaler": {},
        "sklearn.preprocessing.MinMaxScaler": {},
        "sklearn.preprocessing.Normalizer": {"norm": ["l1", "l2", "max"]},
        "sklearn.kernel_approximation.Nystroem": {
            "kernel": [
                "rbf",
                "cosine",
                "chi2",
                "laplacian",
                "polynomial",
                "poly",
                "linear",
                "additive_chi2",
                "sigmoid",
            ],
            "gamma": np.arange(0.0, 1.01, 0.05),
            "n_components": range(1, 11),
        },
        "sklearn.decomposition.PCA": {
            "svd_solver": ["randomized"],
            "iterated_power": range(1, 11),
        },
        "sklearn.preprocessing.PolynomialFeatures": {
            "degree": [2],
            "include_bias": [False],
            "interaction_only": [False],
        },
        "sklearn.kernel_approximation.RBFSampler": {
            "gamma": np.arange(0.0, 1.01, 0.05)
        },
        "sklearn.preprocessing.RobustScaler": {},
        "sklearn.preprocessing.StandardScaler": {},
        "tpot.builtins.ZeroCount": {},
        "tpot.builtins.OneHotEncoder": {
            "minimum_fraction": [0.05, 0.1, 0.15, 0.2, 0.25],
            "sparse": [False],
            "threshold": [10],
        },
        # Selectors
        "sklearn.feature_selection.SelectFwe": {
            "alpha": np.arange(0, 0.05, 0.001),
            "score_func": {"sklearn.feature_selection.f_regression": None},
        },
        "sklearn.feature_selection.SelectPercentile": {
            "percentile": range(1, 100),
            "score_func": {"sklearn.feature_selection.f_regression": None},
        },
        "sklearn.feature_selection.VarianceThreshold": {
            "threshold": [0.0001, 0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.2]
        },
        "sklearn.feature_selection.SelectFromModel": {
            "threshold": np.arange(0, 1.01, 0.05),
            "estimator": {
                "sklearn.ensemble.ExtraTreesRegressor": {
                    "n_estimators": [100],
                    "max_features": np.arange(0.05, 1.01, 0.05),
                }
            },
        },
    }

    classifier_config_dict = {
        # Classifiers
        "sklearn.naive_bayes.GaussianNB": {},
        "sklearn.naive_bayes.BernoulliNB": {
            "alpha": [1e-3, 1e-2, 1e-1, 1.0, 10.0, 100.0],
            "fit_prior": [True, False],
        },
        "sklearn.naive_bayes.MultinomialNB": {
            "alpha": [1e-3, 1e-2, 1e-1, 1.0, 10.0, 100.0],
            "fit_prior": [True, False],
        },
        "sklearn.tree.DecisionTreeClassifier": {
            "criterion": ["gini", "entropy"],
            "max_depth": range(1, 11),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
        },
        "sklearn.ensemble.ExtraTreesClassifier": {
            "n_estimators": [100],
            "criterion": ["gini", "entropy"],
            "max_features": np.arange(0.05, 1.01, 0.05),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
            "bootstrap": [True, False],
        },
        "sklearn.ensemble.RandomForestClassifier": {
            "n_estimators": [100],
            "criterion": ["gini", "entropy"],
            "max_features": np.arange(0.05, 1.01, 0.05),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
            "bootstrap": [True, False],
        },
        "sklearn.ensemble.GradientBoostingClassifier": {
            "n_estimators": [100],
            "learning_rate": [1e-3, 1e-2, 1e-1, 0.5, 1.0],
            "max_depth": range(1, 11),
            "min_samples_split": range(2, 21),
            "min_samples_leaf": range(1, 21),
            "subsample": np.arange(0.05, 1.01, 0.05),
            "max_features": np.arange(0.05, 1.01, 0.05),
        },
        "sklearn.neighbors.KNeighborsClassifier": {
            "n_neighbors": range(1, 101),
            "weights": ["uniform", "distance"],
            "p": [1, 2],
        },
        "sklearn.svm.LinearSVC": {
            "penalty": ["l1", "l2"],
            "loss": ["hinge", "squared_hinge"],
            "dual": [True, False],
            "tol": [1e-5, 1e-4, 1e-3, 1e-2, 1e-1],
            "C": [1e-4, 1e-3, 1e-2, 1e-1,
                  0.5, 1.0, 5.0, 10.0, 15.0, 20.0, 25.0],
        },
        "sklearn.linear_model.LogisticRegression": {
            "penalty": ["l1", "l2"],
            "C": [1e-4, 1e-3, 1e-2, 1e-1, 0.5,
                  1.0, 5.0, 10.0, 15.0, 20.0, 25.0],
            "dual": [True, False],
        },
        "xgboost.XGBClassifier": {
            "n_estimators": [100],
            "max_depth": range(1, 11),
            "learning_rate": [1e-3, 1e-2, 1e-1, 0.5, 1.0],
            "subsample": np.arange(0.05, 1.01, 0.05),
            "min_child_weight": range(1, 21),
            "nthread": [1],
        },
        # Preprocesssors
        "sklearn.preprocessing.Binarizer":
        {"threshold": np.arange(0.0, 1.01, 0.05)},
        "sklearn.decomposition.FastICA": {"tol": np.arange(0.0, 1.01, 0.05)},
        "sklearn.cluster.FeatureAgglomeration": {
            "linkage": ["ward", "complete", "average"],
            "affinity": ["euclidean", "l1", "l2", "manhattan", "cosine"],
        },
        "sklearn.preprocessing.MaxAbsScaler": {},
        "sklearn.preprocessing.MinMaxScaler": {},
        "sklearn.preprocessing.Normalizer": {"norm": ["l1", "l2", "max"]},
        "sklearn.kernel_approximation.Nystroem": {
            "kernel": [
                "rbf",
                "cosine",
                "chi2",
                "laplacian",
                "polynomial",
                "poly",
                "linear",
                "additive_chi2",
                "sigmoid",
            ],
            "gamma": np.arange(0.0, 1.01, 0.05),
            "n_components": range(1, 11),
        },
        "sklearn.decomposition.PCA": {
            "svd_solver": ["randomized"],
            "iterated_power": range(1, 11),
        },
        "sklearn.preprocessing.PolynomialFeatures": {
            "degree": [2],
            "include_bias": [False],
            "interaction_only": [False],
        },
        "sklearn.kernel_approximation.RBFSampler": {
            "gamma": np.arange(0.0, 1.01, 0.05)
        },
        "sklearn.preprocessing.RobustScaler": {},
        "sklearn.preprocessing.StandardScaler": {},
        "tpot.builtins.ZeroCount": {},
        "tpot.builtins.OneHotEncoder": {
            "minimum_fraction": [0.05, 0.1, 0.15, 0.2, 0.25],
            "sparse": [False],
            "threshold": [10],
        },
        # Selectors
        "sklearn.feature_selection.SelectFwe": {
            "alpha": np.arange(0, 0.05, 0.001),
            "score_func": {"sklearn.feature_selection.f_classif": None},
        },
        "sklearn.feature_selection.SelectPercentile": {
            "percentile": range(1, 100),
            "score_func": {"sklearn.feature_selection.f_classif": None},
        },
        "sklearn.feature_selection.VarianceThreshold": {
            "threshold": [0.0001, 0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.2]
        },
        "sklearn.feature_selection.RFE": {
            "step": np.arange(0.05, 1.01, 0.05),
            "estimator": {
                "sklearn.ensemble.ExtraTreesClassifier": {
                    "n_estimators": [100],
                    "criterion": ["gini", "entropy"],
                    "max_features": np.arange(0.05, 1.01, 0.05),
                }
            },
        },
        "sklearn.feature_selection.SelectFromModel": {
            "threshold": np.arange(0, 1.01, 0.05),
            "estimator": {
                "sklearn.ensemble.ExtraTreesClassifier": {
                    "n_estimators": [100],
                    "criterion": ["gini", "entropy"],
                    "max_features": np.arange(0.05, 1.01, 0.05),
                }
            },
        },
    }
    return regressor_config_dict, classifier_config_dict



"""Module to process spectrum like data."""

import numpy as np
from jarvis.core.utils import lorentzian
from scipy.signal import find_peaks_cwt


class Spectrum(object):
    """Module for spectrum like data, e.g. IR, Raman, DOS, epsilon."""

    def __init__(self, x=[], y=[], linewidth=5.0, resolution=0.1):
        """Initialize the class."""
        self.x = np.array(x)
        self.y = np.array(y)
        self.resolution = resolution
        self.linewidth = linewidth

    def rescale(self, mode="max", scaling_factor=1.0):
        """Rescale the spectrum."""
        if mode == "sum":
            const = np.sum(self.y, axis=0)
        if mode == "max":
            const = np.max(self.y, axis=0)
        return self.y * scaling_factor / const

    @property
    def num_modes(self):
        """Get number of modes."""
        return len(self.x)

    @property
    def min_x(self):
        """Get minimum mode frequency."""
        return min(self.x)

    @property
    def max_x(self):
        """Get maximum mode frequency."""
        return max(self.x)

    def get_peak_indices(self, window=np.arange(1, 10)):
        """Get peak indices for non-zero peaks."""
        return find_peaks_cwt(self.y, window)

    def smoothen_spiky_spectrum(self):
        """Smoothen peak for delta function like peaks."""
        lwidth_list = [float(self.linewidth) for i in range(self.num_modes)]
        spect_x = np.arange(
            self.min_x,
            self.max_x + self.resolution,
            self.resolution,
            dtype=np.float64,
        )
        spect_y = np.zeros_like(spect_x, dtype=np.float64)
        for i, j, k in zip(self.x, self.y, lwidth_list):
            spect_y += lorentzian(spect_x, j, i, k)
        return spect_x, spect_y

    def get_interpolated_values(self, new_dist=np.arange(0, 15, 0.05)):
        """Get interpolated grid on a fixed grid."""
        interp = np.interp(new_dist, self.x, self.y)
        return interp


"""
from jarvis.io.vasp.outputs import Vasprun, Outcar
from jarvis.analysis.phonon.ir import ir_intensity

import os

out = Outcar(
    os.path.join(os.path.dirname(__file__),
     "../tests/testfiles/io/vasp/OUTCAR.JVASP-39")
)
vrun = Vasprun(
    os.path.join(
        os.path.dirname(__file__),
        "../tests/testfiles/io/vasp/vasprun.xml.JVASP-39"
    )
)


phonon_eigenvectors = vrun.dfpt_data["phonon_eigenvectors"]
vrun_eigs = vrun.dfpt_data["phonon_eigenvalues"]
phonon_eigenvalues = out.phonon_eigenvalues
masses = vrun.dfpt_data["masses"]
born_charges = vrun.dfpt_data["born_charges"]
x, y = ir_intensity(
    phonon_eigenvectors=phonon_eigenvectors,
    phonon_eigenvalues=phonon_eigenvalues,
    masses=masses,
    born_charges=born_charges,
)
assert x[0] == 713.8676686817399
sp=Spectrum(x=x,y=y)
xx,yy=sp.smoothen_spiky_spectrum()
print (sp.get_peak_indices())
#print (xx.tolist())
print ()
#print (yy.tolist())
"""



"""Class for writing Wannier90 input files."""

import os
import json
import shutil
from collections import OrderedDict
from jarvis.core.atoms import Atoms


class Wannier90win(object):
    """Class for writing wannier90.win."""

    def __init__(
        self,
        struct="",
        dis_num_iter=100,
        kmesh_tol=0.000000001,
        dis_mix_ratio=0.5,
        num_iter=100,
        num_print_cycles=50,
        frozen_tol=2,
        efermi=None,
        soc=True,
        semi_core_states=None,
    ):
        """

        Provide information ndeeded for writing wannier90.win.

        At least Atoms object needed.

        Args:
            struct : Atoms object

            efermi : Fermi-energy

            soc : spin-orbit coupling True/False

            dis_num_iter : number of disentanglement iteration steps
                           generally 100-200 is enough

            num_iter : number of wannierisation iterations

            kmesh_tol : important for wannier convergence,
                        try to increase/decrease if
                        wannierization fails

            dis_mix_ratio : mixing ratio for the disentanglement routine

            frozen_tol :  disentanglement inner (frozen) window with +/- values

            semi_core_states : important to specify according to
                              pseudopotentials. A generic one is
                              given for GGA-PBE, change if you use
                              different POTCARs
        """
        self.struct = struct
        self.efermi = efermi
        self.soc = soc
        self.dis_num_iter = dis_num_iter
        self.dis_mix_ratio = dis_mix_ratio
        self.num_iter = num_iter
        self.num_print_cycles = num_print_cycles
        self.frozen_tol = frozen_tol
        self.semi_core_states = semi_core_states
        self.kmesh_tol = kmesh_tol
        if self.semi_core_states is None:
            path_semi_core = str(
                os.path.join(
                    os.path.dirname(__file__), "default_semicore.json"
                )
            )
            f = open(path_semi_core, "r")
            semi_core_states = json.load(f)
            f.close()
            self.semi_core_states = semi_core_states

    def to_dict(self):
        """Convert to a dictionary."""
        d = OrderedDict()
        d["struct"] = self.struct.to_dict()
        d["efermi"] = self.efermi
        d["soc"] = self.soc
        d["dis_num_iter"] = self.dis_num_iter
        d["dis_mix_ratio"] = self.dis_mix_ratio
        d["num_iter"] = self.num_iter
        d["num_print_cycles"] = self.num_print_cycles
        d["frozen_tol"] = self.frozen_tol
        d["semi_core_states"] = self.semi_core_states
        d["kmesh_tol"] = self.kmesh_tol
        return d

    @classmethod
    def from_dict(self, d={}):
        """Convert class from a dictionary."""
        return Wannier90win(
            struct=Atoms.from_dict(d["struct"]),
            efermi=d["efermi"],
            soc=d["soc"],
            dis_num_iter=d["dis_num_iter"],
            dis_mix_ratio=d["dis_mix_ratio"],
            num_iter=d["num_iter"],
            num_print_cycles=d["num_print_cycles"],
            frozen_tol=d["frozen_tol"],
            semi_core_states=d["semi_core_states"],
            kmesh_tol=d["kmesh_tol"],
        )

    def write_win(self, name="win.input"):
        """Write .win file."""
        if self.soc:
            mult = 2
        else:
            mult = 1
        semi_core = self.semi_core_states

        xx = self.struct.composition.to_dict()
        elements = []
        ELEMENTCOUNT = []
        for i, j in xx.items():
            elements.append(i)
            ELEMENTCOUNT.append(int(j))
        nwan = 0
        nelectrons = 0
        exclude = 0
        projections = ""
        c = 0
        for e in elements:
            s = semi_core[e]
            exclude += s[0] * ELEMENTCOUNT[c]
            nwan += mult * s[3] * ELEMENTCOUNT[c]
            nelectrons += s[1] * ELEMENTCOUNT[c]
            projections += e + ":" + s[2] + "\n"
            c += 1
        print("excluded", exclude)
        nwan_tot = nwan + exclude
        print("nwan_tot", nwan_tot)
        print("projections", projections)
        if exclude == 0:
            exclude_st = ""
        else:
            exclude_st = "exclude_bands     = 1-" + str(int(exclude)) + "\n"

        f = open(name, "w")
        line = "guiding_centres = TRUE \n"
        f.write(line)
        line = "kmesh_tol=" + str(self.kmesh_tol) + "\n"
        f.write(line)
        # line=str('num_wann =')+str(nwan_tot)+'\n'
        line = str("num_wann =") + str(nwan) + "\n"
        f.write(line)
        f.write(exclude_st)

        tol = self.frozen_tol
        if self.efermi is not None:
            dmax = self.efermi + tol
            # dmin = self.efermi - tol
            line = str("dis_froz_max =") + str(dmax) + str("\n")
            f.write(line)
            line = str("dis_froz_min =") + str(-1000) + str("\n")
            # line = str("dis_froz_min =") + str(dmin) + str("\n")
            f.write(line)

        line = str("dis_num_iter =") + str(self.dis_num_iter) + str("\n")
        f.write(line)
        line = str("dis_mix_ratio =") + str(self.dis_mix_ratio) + str("\n")
        f.write(line)
        line = str("num_iter =") + str(self.num_iter) + str("\n")
        f.write(line)
        line = (
            str("num_print_cycles =") + str(self.num_print_cycles) + str("\n")
        )
        f.write(line)
        line = str("begin projections") + str("\n")
        f.write(line)
        f.write(projections)
        line = str("end projections") + str("\n")
        f.write(line)
        f.close()
        return nwan, exclude

    def write_hr_win(
        self,
        hr_tag="hr_plot",
        prev_win="wannier90.win",
        hr="hr_wannier.win",
        nbands=18,
        soc="",
    ):
        """Write hr_plot or write_hr as .true."""
        f = open(prev_win, "r")
        lines = f.read().splitlines()
        f.close()
        bak = str(prev_win) + str(".bak")
        shutil.copy2(prev_win, bak)

        # line=str('write_hr=.true. \n')
        line = str(hr_tag) + str("=.true. \n")

        f = open(hr, "w")
        f.write(line)
        line = str("num_bands = ") + str(nbands) + ("\n")
        f.write(line)
        for i in lines:
            line = str(i) + str("\n")
            f.write(line)
        f.close()


"""
if __name__ == "__main__":
    from jarvis.core.atoms import Atoms
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    Wannier90win(struct=Si, efermi=0.0).write_win()
    Wannier90win().write_hr_win(prev_win="win.input")
"""



"""Class for writing inputs for BoltzTrap calculations."""
import numpy as np
from jarvis.io.vasp.outputs import Vasprun
from jarvis.analysis.structure.spacegroup import Spacegroup3D

Ry_to_ev = 13.6056980659
Angs_to_Bohr = 1.88973


class WriteInputs(object):
    """Write input files for BoltzTrap."""

    def __init__(
        self,
        vasprun_path="",
        dos_type="HISTO",
        tmax=1300,
        tgrid=50,
        doping=[],
        run_type="BOLTZ",
        symprec=1e-2,
        energy_grid=0.005,
        lpfac=10,
        energy_span_around_fermi=1.5,
        energy=None,
        struct=None,
        intrans=None,
    ):
        """
        Require following information.

        energy: energy window.

        struct: Atoms object.

        intrans: name of intrans.

        vasprun_path:  path of vasprun file.

        dos_type: type of densit of states.

        tmax: maximum temperature.

        tgrid: temperature grid.

        doping: doping levels

        run_type:

        symprec: symmetry precision.

        energy_grid: energy grid.

        lpfac:

        energy_span_around_fermi:

        """
        self.energy = energy
        self.struct = struct
        self.intrans = intrans
        self.vasprun_path = vasprun_path
        self.vrun = Vasprun(filename=vasprun_path)
        self.energy_grid = energy_grid
        self.lpfac = lpfac
        self.run_type = run_type
        self.energy_span_around_fermi = energy_span_around_fermi
        self.tmax = tmax
        self.tgrid = tgrid
        self.dos_type = dos_type
        self.doping = doping
        if self.doping == []:
            for d in [1e16, 1e17, 1e18, 1e19, 1e20, 1e21]:
                self.doping.extend([1 * d, 2.5 * d, 5 * d, 7.5 * d])
            self.doping.append(1e22)

    def write_intrans(self, filename="boltztrap.intrans"):
        """Write BoltzTrap input intrans file."""
        f = open(filename, "w")
        scissor = 0.0
        nelect = int(float(self.vrun.all_input_parameters["NELECT"]))
        setgap = 1 if scissor > 0.0001 else 0
        f.write("GENE          # use generic interface\n")
        f.write(
            "1 0 %d %f         # iskip (not presently used) idebug "
            "setgap shiftgap \n" % (setgap, scissor / float(Ry_to_ev))
        )
        f.write(
            "0.0 %f %f %6.1f     # Fermilevel (Ry),energygrid,energy "
            "span around Fermilevel, number of electrons\n"
            % (
                self.energy_grid / float(Ry_to_ev),
                self.energy_span_around_fermi / float(Ry_to_ev),
                nelect,
            )
        )
        f.write(
            "CALC                    # CALC (calculate expansion "
            "coeff), NOCALC read from file\n"
        )
        f.write(
            "%d                        # lpfac, number of latt-points "
            "per k-point\n" % self.lpfac
        )
        f.write(
            "%s                     # run mode (only BOLTZ is "
            "supported)\n" % self.run_type
        )

        f.write(
            ".15                       # (efcut) energy range of "
            "chemical potential\n"
        )
        f.write(
            "{} {}                  # Tmax, temperature grid\n".format(
                self.tmax, self.tgrid
            )
        )

        f.write(
            "-1.  # energyrange of bands given DOS output sig_xxx and "
            "dos_xxx (xxx is band number)\n"
        )

        f.write(self.dos_type + "\n")
        self.taurf = 0
        self.tauexp = 0
        self.tauen = 0
        f.write("{} {} {} 0 0 0\n".format(self.taurf, self.tauexp, self.tauen))

        f.write("{}\n".format(2 * len(self.doping)))
        for i in self.doping:
            f.write(str(i) + "\n")
        for i in self.doping:
            f.write(str(-i) + "\n")
        f.close()

    def write_struct(self, filename="boltztrap.struct"):
        """Write BoltzTrap based struct file."""
        atoms = self.vrun.all_structures[-1]
        spg = Spacegroup3D(atoms)
        spg_symb = spg.space_group_symbol
        formula = atoms.composition.formula
        operations = spg._dataset["rotations"]
        lattice_mat = np.array(atoms.lattice_mat) * Angs_to_Bohr
        f = open(filename, "w")
        f.write("%s %s\n" % (formula, spg_symb))
        f.write(
            "%12.8f %12.8f %12.8f\n"
            % (lattice_mat[0][0], lattice_mat[0][1], lattice_mat[0][2])
        )
        f.write(
            "%12.8f %12.8f %12.8f\n"
            % (lattice_mat[1][0], lattice_mat[1][1], lattice_mat[1][2])
        )
        f.write(
            "%12.8f %12.8f %12.8f\n"
            % (lattice_mat[2][0], lattice_mat[2][1], lattice_mat[2][2])
        )
        f.write("%d\n" % (len(operations)))
        for c in operations:
            for row in c:
                f.write("{}\n".format(" ".join(str(i) for i in row)))
        f.close()

    def write_energy(self, filename="boltztrap.energyso", trim=0.1):
        """Write energy information from DFT."""
        kpoints = self.vrun.kpoints._kpoints
        eigs_up, eigs_dn = self.vrun.eigenvalues
        ef = self.vrun.efermi
        target = 2 * int(len(eigs_dn[0]) * (1 - trim))  # +1
        # print("target", target)
        f = open(filename, "w")
        line = str("system \n") + str(len(kpoints)) + "\n"
        f.write(line)
        for i, j, k in zip(kpoints, eigs_up, eigs_dn):
            count = 0
            line = (" ".join(map(str, i))) + str(" ") + str(target) + "\n"
            # f.write(line)
            f.write("%12.8f %12.8f %12.8f %d\n" % (i[0], i[1], i[2], target))
            for m, n in zip(j, k):
                count = count + 2
                if count <= target:
                    en_up = round((m[0] - ef) / float(Ry_to_ev), 8)
                    en_dn = round((n[0] - ef) / float(Ry_to_ev), 8)
                    f.write("%18.8f\n" % (en_up))
                    f.write("%18.8f\n" % (en_dn))
        f.close()


"""
if __name__ == "__main__":
    from jarvis.io.vasp.outputs import Vasprun

    vrun = Vasprun(
        "/rk2/knc6/JARVIS-DFT/Elements-bulkk/mp-149_bulk_PBEBO/MAIN-RELAX-bulk@mp-149/vasprun.xml"
    )
    print(vrun.final_energy)
    inp = WriteInputs(
        vasprun_path="/rk2/knc6/JARVIS-DFT/Elements-bulkk/mp-149_bulk_PBEBO/MAIN-RELAX-bulk@mp-149/vasprun.xml"
    )
    inp.write_energy()
    inp.write_struct()
    inp.write_intrans()
    cmd = "~/anaconda2/bin/x_trans BoltzTraP -so"
    os.system(cmd)
"""



"""Modules handling chemical composition."""
# from math import gcd
import string
from jarvis.core.specie import Specie
from collections import OrderedDict
from collections import defaultdict
from jarvis.core.utils import gcd
import re
import numpy as np


class Composition(object):
    """Generate composition python object."""

    def __init__(self, content={}, sort=False):
        """
        Initialize with a dictionary.

        This class is used in Atoms and related objects.

        Args:
            content: dictioary.

            sort: whether to sort alphabetically.

        >>> from composition import Composition
        >>> comp = {"Li": 2, "O": 4}
        >>> cc = Composition(comp)
        >>> weight = round(cc.weight,4)
        >>> print(cc.prototype,cc.formula,cc.reduced_formula,weight)
        AB2 Li2O4 LiO2 77.8796
        """
        if sort:
            content = OrderedDict(
                sorted(content.items(), key=lambda x: (x[0]))
            )
        self._content = content

    @staticmethod
    def from_string(value, sort=True):
        """Generate composition from string."""
        re_formula = re.compile(r"([A-Z][a-z]?)([0-9\.]*)")
        d = defaultdict(float)
        for elt, amt in re_formula.findall(value):
            if elt in ["D", "T"]:
                elt = "H"
            if amt == "":
                d[elt] = 1
            elif float(amt).is_integer():
                d[elt] += int(float(amt))
            else:
                d[elt] += float(amt)

        comp = Composition(dict(d), sort=sort)
        return comp

    def reduce(self):
        """Reduce chemical formula."""
        repeat = 0
        for specie, count in self._content.items():
            if repeat == 0:
                repeat = count
            else:
                repeat = gcd(count, repeat)
        reduced = {}
        for specie, count in self._content.items():
            reduced[specie] = count // repeat
        return reduced, repeat

    @property
    def prototype(self):
        """Get chemical prototypes such as A, AB etc."""
        proto = ""
        all_upper = string.ascii_uppercase

        reduced, repeat = self.reduce()
        N = 0
        for specie, count in reduced.items():
            proto = proto + str(all_upper[N]) + str(round(count, 3))
            N = N + 1
        return proto.replace("1", "")

    def to_dict(self):
        """Return dictionary format."""
        return self._content

    @property
    def nspecies(self):
        """Return number of species."""
        return len(list(self.to_dict().keys()))

    @property
    def search_string(self):
        """Return JARVIS sorted search string."""
        return "-".join(sorted(list(self.to_dict().keys())))

    @classmethod
    def from_dict(self, d={}):
        """Load the class from a dictionary."""
        return Composition(content=d)

    @property
    def reduced_formula(self):
        """Get reduced formula."""
        form = ""
        reduced, repeat = self.reduce()
        for specie, count in reduced.items():
            if float(count).is_integer():
                form = form + specie + str(int(count))
            else:
                form = form + specie + str(count)

        return form.replace("1", "")

    @property
    def formula(self):
        """Get total chemical formula."""
        form = ""
        for specie, count in self._content.items():
            if float(count).is_integer():
                form = form + str(specie) + str(int(count))
            else:
                form = form + str(specie) + str(count)
        return form.replace("1", "")

    @property
    def atomic_fraction(self):
        """Get atomic fraction."""
        comp_dict = self.to_dict()
        tot = sum(comp_dict.values())
        new_dict = OrderedDict()
        for i, j in comp_dict.items():
            new_dict[i] = j / tot
        return new_dict

    @property
    def atomic_fraction_array(self):
        """Get atomic fraction array."""
        nelements = len(list(Specie()._data.keys()))
        frac_arr = np.zeros(nelements)
        fracs = self.atomic_fraction
        for i, j in fracs.items():
            frac_arr[int(Specie(i).Z) - 1] = j
        return frac_arr

    @property
    def weight(self):
        """Get atomic weight."""
        wt = 0.0
        for specie, count in self._content.items():
            wt = wt + Specie(specie).atomic_mass * count
        return wt

    def __repr__(self):
        """Representation of the class."""
        return str(self._content)


"""
if __name__ == "__main__":
    c=Composition.from_string('Al2O3Al5Co6O1')
    print ('Al2O3Al5Co6O1',c.formula, c.reduced_formula)
    import sys
    sys.exit()
    comp = {"Li": 2, "O": 4}
    cc = Composition(comp)
    print("dict", cc.to_dict())
    x, y = cc.reduce()
    # print(x, y)
    proto = cc.prototype
    print(proto, cc.formula, cc.reduced_formula)
"""



from jarvis.analysis.structure.neighbors import NeighborsAnalysis
from jarvis.core.atoms import Atoms
import numpy as np
import os
import matplotlib.pyplot as plt
plt.switch_backend('agg')

def test_nbors():
    box = [[5.493642, 0, 0], [0, 5.493642, 0], [0, 0, 5.493642]]
    elements = ["Si", "Si", "Si", "Si", "Si", "Si", "Si", "Si"]
    coords = [
        [0, 0, 0],
        [0.25, 0.25, 0.25],
        [0.000000, 0.500000, 0.500000],
        [0.250000, 0.750000, 0.750000],
        [0.500000, 0.000000, 0.500000],
        [0.750000, 0.250000, 0.750000],
        [0.500000, 0.500000, 0.000000],
        [0.750000, 0.750000, 0.250000],
    ]
    coords = np.array(coords)
    # box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    # coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    # elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    nbr = NeighborsAnalysis(Si)
    nb = nbr.get_all_distributions
    tmp = round((nb["rdf"][-3]), 2)
    assert (tmp) == (4.08)
    nbr.get_rdf(plot=True)
    # nbr.ang_dist(nbor_info=info,plot=True)
    nbr.ang_dist_first(plot=True)
    nbr.ang_dist_second(plot=True)
    nbr.get_ddf(plot=True)
    angs = nbr.atomwise_angle_dist()
    ardf = nbr.atomwise_radial_dist()
    cmd = "rm *.png"
    os.system(cmd)



"""Module for writing LAMMPS input files."""

import numpy as np
import pprint
from collections import OrderedDict
from jarvis.core.atoms import Atoms
from jarvis.core.specie import Specie


class LammpsData(object):
    """Construct Lammps data file."""

    def __init__(
        self,
        lammps_box=[],
        species=[],
        charges=[],
        cart_coords=[],
        element_order=[],
    ):
        """Provide following information."""
        self._lammps_box = lammps_box
        self._species = species
        self._charges = charges
        self._cart_coords = cart_coords
        self._element_order = element_order

    def lammps_to_atoms(self):
        """Convert Lammps data to Atoms object."""
        # n_atoms = len(self._species)
        # n_atom_types = len(self._element_order)
        box = self._lammps_box
        xhi = box[1]
        xlo = box[0]
        yhi = box[3]
        ylo = box[2]
        zhi = box[5]
        zlo = box[4]
        xy = box[6]
        xz = box[7]
        yz = box[8]
        lat = np.array(
            [[xhi - xlo, 0.0, 0.0], [xy, yhi - ylo, 0.0], [xz, yz, zhi - zlo]]
        )
        elements = [self._element_order[i - 1] for i in self._species]
        atoms = Atoms(
            lattice_mat=lat,
            elements=elements,
            coords=self._cart_coords,
            cartesian=True,
        )
        return atoms

    def atoms_to_lammps(self, atoms, origin=(0, 0, 0)):
        """Convert Atoms object to Lammps data."""
        # n_atoms = len(self._species)
        elements = atoms.elements
        # num_atoms = atoms.num_atoms
        a, b, c = atoms.lattice.abc
        xlo, ylo, zlo = origin
        xhi = a + xlo
        m = atoms.lattice._lat
        xy = np.dot(m[1], m[0] / a)
        yhi = np.sqrt(b ** 2 - xy ** 2) + ylo
        xz = np.dot(m[2], m[0] / a)
        yz = (np.dot(m[1], m[2]) - xy * xz) / (yhi - ylo)
        zhi = np.sqrt(c ** 2 - xz ** 2 - yz ** 2) + zlo
        rot_matrix = np.linalg.solve(
            [[xhi - xlo, 0, 0], [xy, yhi - ylo, 0], [xz, yz, zhi - zlo]], m
        )
        # box = np.array([[xlo, xhi], [ylo, yhi], [zlo, zhi], [xy, xz, yz]])
        box = np.array([xlo, xhi, ylo, yhi, zlo, zhi, xy, xz, yz])
        new_coords = (np.dot(rot_matrix, atoms.cart_coords.T)).T
        unique_elements = list(set(elements))
        # n_atom_types = len(unique_elements)
        n_atoms = len(elements)
        if self._charges == []:
            self._charges = np.zeros(n_atoms)
        lmp_species = []
        for i, ii in enumerate(elements):
            s = unique_elements.index(ii) + 1
            lmp_species.append(s)
        lmp_species = np.array(lmp_species, dtype="int")
        return LammpsData(
            lammps_box=box,
            species=lmp_species,
            charges=self._charges,
            cart_coords=new_coords,
            element_order=unique_elements,
        )

    def read_data(
        self,
        filename="lammps.data",
        element_order=[],
        potential_file="pot.mod",
        verbose=True,
    ):
        """Read Lammps data file."""
        # n_atoms = len(self._species)
        if element_order == []:
            # Reading potential file for element order
            if verbose:
                print("element_order is empty, reading from", potential_file)
            pot_file = open(potential_file, "r")
            lines = pot_file.read().splitlines()
            pot_file.close()
            symb = []
            # count = 0

            for i, line in enumerate(lines):
                if "pair_coeff" in line.split():
                    sp = line.split()
                    # print("spsplit", sp, os.getcwd())
                    for el in sp:
                        try:
                            if str(Specie(el).Z) != "nan":
                                # if el=='M':
                                #    el='Mo'
                                # count=count+1
                                # if count >4:
                                symb.append(Specie(el).symbol)
                        except Exception:
                            pass
        else:
            symb = [Specie(i).symbol for i in element_order]

        # print("symb=", symb)
        f = open(filename, "r")
        lines = f.read().splitlines()
        for i, line in enumerate(lines):
            if "atoms" in line.split():
                natoms = int(line.split()[0])
            if "types" in line.split():
                # print(line)
                ntypes = int(line.split()[0])
            if "xlo" in line.split():
                xlo = float(line.split()[0])
                xhi = float(line.split()[1])
            if "ylo" in line.split():
                ylo = float(line.split()[0])
                yhi = float(line.split()[1])
            if "zlo" in line.split():
                zlo = float(line.split()[0])
                zhi = float(line.split()[1])
            if "xy" in line.split():
                xy = float(line.split()[0])
                xz = float(line.split()[1])
                yz = float(line.split()[2])
        if len(symb) != ntypes:
            ValueError(
                "Something wrong in atom type assignment", len(symb), ntypes
            )
        lat = np.array(
            [[xhi - xlo, 0.0, 0.0], [xy, yhi - ylo, 0.0], [xz, yz, zhi - zlo]]
        )
        typ = np.empty((natoms), dtype="S20")
        x = np.zeros((natoms))
        y = np.zeros((natoms))
        z = np.zeros((natoms))
        q = np.zeros((natoms))
        coords = list()
        for i, line in enumerate(lines):
            if "Atoms" in line.split():
                for j in range(0, natoms):
                    # print int(((lines[j+2]).split()[1]))-1
                    typ[j] = symb[int(((lines[i + j + 2]).split()[1])) - 1]
                    q[j] = (lines[i + j + 2]).split()[2]
                    x[j] = (lines[i + j + 2]).split()[3]
                    y[j] = (lines[i + j + 2]).split()[4]
                    z[j] = (lines[i + j + 2]).split()[5]
                    coords.append([x[j], y[j], z[j]])
        f.close()
        # print ("info",(typ),'coo',(coords),'latt',lat)
        typ_sp = [str(i, "utf-8") for i in typ]
        # print ('typ_sp',typ_sp)
        atoms = Atoms(
            lattice_mat=lat,
            elements=typ_sp,
            coords=np.array(coords),
            cartesian=True,
        )
        return atoms

    def write_file(self, filename="lammps.data"):
        """Write Lammps data input file."""
        # n_atoms = len(self._species)
        n_atoms = len(self._species)
        n_atom_types = len(self._element_order)
        box = self._lammps_box
        xhi = box[1]
        xlo = box[0]
        yhi = box[3]
        ylo = box[2]
        zhi = box[5]
        zlo = box[4]
        xy = box[6]
        xz = box[7]
        yz = box[8]

        f = open(filename, "w")
        f.write("datafile (written by JARVIS-Tools) \n\n")
        f.write("%d \t atoms \n" % n_atoms)
        f.write("%d  atom types\n" % n_atom_types)
        f.write("%s %s  xlo xhi\n" % (xlo, xhi))
        f.write("%s %s  ylo yhi\n" % (ylo, yhi))
        f.write("%s %s  zlo zhi\n" % (zlo, zhi))
        f.write("%s %s %s  xy xz yz\n" % (xy, xz, yz))
        f.write("\n\n")
        f.write("Atoms \n\n")
        for i in range(n_atoms):
            s = self._species[i]
            r = self._cart_coords[i]
            charge = self._charges[i]
            f.write(
                "%6d %3d %6f %s %s %s\n" % (i + 1, s, charge, r[0], r[1], r[2])
            )
        f.close()

    def to_dict(self):
        """Convert the infor to a dictionary."""
        d = OrderedDict()
        d["box"] = self._lammps_box
        d["species"] = self._species
        d["charges"] = self._charges
        d["cart_coords"] = self._cart_coords
        d["element_order"] = self._element_order
        return d

    @classmethod
    def from_dict(self, d={}):
        """Construct from a dictionary."""
        return LammpsData(
            lammps_box=d["box"],
            species=d["species"],
            charges=d["charges"],
            cart_coords=d["cart_coords"],
            element_order=d["element_order"],
        )

    def __repr__(self, indent=4):
        """Print method."""
        return pprint.pformat(self.to_dict(), indent=indent)


class LammpsInput(object):
    """Construct LAMMPS input."""

    def __init__(self, LammpsDataObj=None, pbc=["p", "p", "p"]):
        """Require LammpsData class and periodic boundary conditions."""
        self.LammpsDataObj = LammpsDataObj
        self.pbc = pbc

    def to_dict(self):
        """Convert to a dictionary."""
        d = OrderedDict()
        d["LammpsDataObj"] = self.LammpsDataObj.to_dict()
        d["pbc"] = self.pbc
        return d

    @classmethod
    def from_dict(self, d={}):
        """Costruct class from a dictionary."""
        return LammpsInput(
            LammpsDataObj=LammpsData.from_dict(d["LammpsDataObj"]),
            pbc=d["pbc"],
        )

    def write_lammps_in(
        self,
        lammps_in="init.mod",
        lammps_in1="potential.mod",
        lammps_in2="in.main",
        lammps_trj=None,
        lammps_data=None,
        parameters={},
    ):
        """
        Write lammps input file.

        From ase with custom modifications
        LAMMPS input is devided into three parts

        Args:
            lammps_in: generally"init.mod",
            with unit and conversion factor information

            lammps_in1: generally "potential.mod",
            with force-field/potential style and
            element type information

            lammps_in2: generally "in.elastic",
            a generic main input file to be fed
            in LAMMPS usin lmp_*<...,parameters['exec']

            parameters: input parameters

        """
        f = open(lammps_in, "w")
        f1 = open(lammps_in1, "w")  # potential.mod
        f2 = open(lammps_in2, "w")
        f.write(
            ('variable dump_file string "%s"\n' % lammps_trj)
            + ("variable up  equal 1.0e-6\n")
            + ("variable cfac  equal 1.0e-4\n")
            + ("variable cunits  string GPa\n")
            + ("variable etol  equal 0\n")
            + ("variable ftol  equal 1.0e-10\n")
            + ("variable maxiter equal 1000\n")
            + ("variable maxeval equal 10000\n")
            + ("variable dmax equal 1.0e-2\n")
            + ('variable data_file string "%s"\n' % "data")
        )
        if "control_file" in parameters:
            f2.write("include %s \n" % parameters["control_file"])
        if "units" in parameters:
            f.write("units %s \n" % parameters["units"])
        else:
            f.write("units metal \n")
        if "atom_style" in parameters:
            f.write("atom_style %s \n" % parameters["atom_style"])
        else:
            f.write("atom_style atomic \n")
        if "boundary" in parameters:
            f.write("boundary %s \n" % parameters["boundary"])
        else:
            pbc = self.pbc
            f.write("boundary %s %s %s \n" % (pbc[0], pbc[1], pbc[2]))
        f.write("atom_modify sort 0 0.0 \n")
        for key in ("neighbor", "newton"):
            if key in parameters:
                f.write("%s %s \n" % (key, parameters[key]))
        f.write("\n")
        f.write("read_data %s\n" % "data")
        f.write("\n### interactions \n")
        if "lib" in parameters:
            lib = parameters["lib"]
            f1.write("%s \n" % lib)
        if ("pair_style" in parameters) and ("pair_coeff" in parameters):
            pair_style = parameters["pair_style"]
            f1.write("pair_style %s \n" % pair_style)
            symbols = self.LammpsDataObj._element_order
            tag = ""
            for i in symbols:
                tag = tag + " " + i
            pair_coef = "* * " + str(parameters["pair_coeff"]) + " " + tag
            f1.write("pair_coeff %s \n" % pair_coef)

            masses = []
            for i in symbols:
                m = Specie(i).atomic_mass
                if m not in masses:
                    masses.append(m)
            count = 0
            for i in masses:
                count = count + 1
                f.write("mass" + " " + str(count) + " " + str(i) + "\n")

        else:

            # default interaction
            f.write(
                "pair_style lj/cut 2.5 \n"
                + "pair_coeff * * 1 1 \n"
                + "mass * 1.0 \n"
            )
        f1.write("neighbor 1.0 nsq\n")
        f1.write("neigh_modify once no every 1 delay 0 check yes\n")
        if "min" not in parameters:
            f1.write("min_style  cg\n")
            f1.write("min_modify           dmax ${dmax} line quadratic\n")
        f1.write("thermo          1\n")
        f1.write(
            "thermo_style custom step temp press cpu pxx pyy pzz\
             pxy pxz pyz ke pe etotal vol lx ly lz atoms\n"
        )
        f1.write("thermo_modify norm no\n")
        if "fix" in parameters:
            if parameters["fix"]:
                for i in parameters["fix"]:
                    f1.write("fix %s\n" % i)
        f.close()
        f1.close()
        f2.close()


"""
if __name__ == "__main__":
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    spg = Spacegroup3D(atoms=Si)
    Si = spg.conventional_standard_structure
    p = Poscar.from_file(
        "/rk2/knc6/JARVIS-FF/FS/Al1.eam.fs_nist/bulk@mp-134_fold/mp-134/new_pymatgen_slab.vasp"
    )
    p = Poscar.from_file(
        "/rk2/knc6/JARVIS-FF/COMB/ffield.comb3.NiAlO_nist/bulk@mp-1143_fold/bulk@mp-1143/new_pymatgen_slab.vasp"
    )
    # print(lmp)
    lmp = LammpsData().atoms_to_lammps(atoms=p.atoms)
    lmp.write_file()
    x = lmp.lammps_to_atoms()
    # print (x)
    lmp = LammpsData().read_data("lammps.data")
    # print(lmp)
    lmp = LammpsData().atoms_to_lammps(atoms=p.atoms)
    pair_coeff = "abc"
    LammpsInput(LammpsDataObj=lmp).write_lammps_in(
        parameters={
            "pair_style": "eam/alloy",
            "pair_coeff": pair_coeff,
            "atom_style": "charge",
            "control_file": "/users/knc6/inelast.mod",
        }
    )
"""



"""Module for processing force-constant."""
import numpy as np


def read_fc(filename="FORCE_CONSTANTS"):
    """Read force-constants."""
    from jarvis.io.phonopy.outputs import read_fc
    fc = read_fc(filename)
    return fc


def qpoint(force_constant=[], qpt=[0.0, 0.0, 0.0]):
    """Get FC as natons x natons x 3 x3."""
    qpt = np.array(qpt)
    exp_iqpt = np.exp(1.0j * qpt)
    dmat = force_constant * exp_iqpt
    vals, vects = np.linalg.eigh(dmat)
    return vals, vects



from jarvis.core.graphs import StructureDataset, Graph
from jarvis.db.figshare import data


def test_graph():
    from jarvis.core.atoms import Atoms
    from jarvis.db.figshare import get_jid_data

    atoms = Atoms.from_dict(get_jid_data("JVASP-664")["atoms"])
    feature_sets = ("atomic_number", "basic", "cfid", "cgcnn")
    for i in feature_sets:
        g = Graph.atom_dgl_multigraph(atoms=atoms, atom_features=i)
        print(i, g)
    g = Graph.from_atoms(atoms=atoms, features="atomic_number")
    g = Graph.from_atoms(atoms=atoms, features="atomic_fraction")
    g = Graph.from_atoms(
        atoms=atoms,
        features="basic",
        get_prim=True,
        zero_diag=True,
        node_atomwise_angle_dist=True,
        node_atomwise_rdf=True,
    )
    g = Graph.from_atoms(
        atoms=atoms,
        features="cfid",
        get_prim=True,
        zero_diag=True,
        node_atomwise_angle_dist=True,
        node_atomwise_rdf=True,
    )
    g = Graph.from_atoms(
        atoms=atoms,
        features="atomic_number",
        get_prim=True,
        zero_diag=True,
        node_atomwise_angle_dist=True,
        node_atomwise_rdf=True,
    )
    g = Graph.from_atoms(atoms=atoms, features="basic")
    g = Graph.from_atoms(
        atoms=atoms, features=["Z", "atom_mass", "max_oxid_s"]
    )
    g = Graph.from_atoms(atoms=atoms, features="cfid", max_cut=10000)
    print(g)
    d = g.to_dict()
    g = Graph.from_dict(d)
    num_nodes = g.num_nodes
    num_edges = g.num_edges
    print(num_nodes, num_edges)
    assert num_nodes == 48
    assert num_edges == 2256
    assert (g.adjacency_matrix.shape) == (48, 48)


def test_dataset():
    d = data("dft_2d")
    x = []
    y = []
    z = []
    for i in d[0:100]:
        if i["formation_energy_peratom"] != "na":
            x.append(i["atoms"])
            y.append(i["formation_energy_peratom"])
            z.append(i["jid"])
    s = StructureDataset(x, y, ids=z)


# test_graph()



"""Modules for job submission."""

import os
import subprocess
from collections import OrderedDict


class Queue(object):
    """Construct HPC Job class."""

    def __init__(
        self,
        q_type="head_node",
        q_parameters={},
        job_sub_cmd=None,
        job_check_cmd="",
        job_id="",
    ):
        """Initialize the class."""
        self.q_type = q_type
        self.q_parameters = q_parameters
        self.job_sub_cmd = job_sub_cmd
        self.job_check_cmd = job_check_cmd
        self.job_id = job_id

    def head_node(self, submit_cmd=None):
        """Select if run on the head node, not recommended."""
        os.system(submit_cmd)

    def to_dict(self):
        """Convert class to a dictionary."""
        d = OrderedDict()
        d["q_type"] = self.q_type
        d["q_parameters"] = self.q_parameters
        d["job_sub_cmd"] = self.job_sub_cmd
        d["job_check_cmd"] = self.job_check_cmd
        d["job_id"] = self.job_id
        return d

    @classmethod
    def from_dict(self, d={}):
        """Load from a dictionary."""
        return Queue(
            q_type=d["q_type"],
            q_parameters=d["q_parameters"],
            job_sub_cmd=d["job_sub_cmd"],
            job_check_cmd=d["job_check_cmd"],
            job_id=d["job_id"],
        )

    @classmethod
    def pbs(
        self,
        filename="submit_job",
        shell="#!/bin/bash",
        nnodes=1,
        cores=16,
        walltime=None,
        queue=None,
        account=None,
        group_name=None,
        jobname="myJob",
        jobout="job.out",
        joberr="job.err",
        memory=None,
        email=None,
        pre_job_lines=None,
        directory=None,
        env=None,
        job_line="echo I am here",
        post_job_lines=None,
        submit_cmd=None,
    ):
        """Select if run using PBS script."""
        f = open(filename, "w")
        f.write("%s\n" % shell)
        f.write("#PBS -l nodes=%d:ppn=%d\n" % (nnodes, cores))
        f.write("#PBS -N %s\n" % jobname)
        f.write("#PBS -o %s\n" % jobout)
        f.write("#PBS -e %s\n" % joberr)

        if walltime is not None:
            if isinstance(walltime, str):
                f.write("#PBS -l walltime=%s\n" % walltime)
            else:
                ValueError("Provide walltime in a string format", walltime)
        if queue is not None:
            f.write("#PBS -q %s\n" % queue)
        if account is not None:
            f.write("#PBS -A %s\n" % account)
        if group_name is not None:
            f.write("#PBS -G %s\n" % group_name)
        if env is not None:
            f.write("#PBS -V %s\n" % env)
        if email is not None:
            f.write("#PBS -M %s\n" % email)
        if memory is not None:
            f.write("#PBS -l %s\n" % memory)
        if pre_job_lines is not None:
            f.write("%s\n" % pre_job_lines)
        if directory is not None:
            f.write("cd %s\n" % directory)
        if job_line is not None:
            f.write("%s\n" % job_line)
        if post_job_lines is not None:
            f.write("%s\n" % post_job_lines)
        f.close()

        if submit_cmd is not None:

            with open("jobid", "w") as f:
                p = subprocess.Popen(
                    submit_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
                )
                stdout, stderr = p.communicate()
                print("stdout,stderr", stdout, stderr)
                # job_id = str(stdout.split('Your job')[1].split(' ')[1])
                f.write(str(stdout))

        print("x")

    @classmethod
    def slurm(
        self,
        filename="submit_job",
        shell="#!/bin/bash",
        nnodes=1,
        cores=16,
        walltime=None,
        queue=None,
        account=None,
        group_name=None,
        jobname="myJob",
        jobout="job.out",
        joberr="job.err",
        memory=None,
        email=None,
        pre_job_lines=None,
        directory=None,
        env=None,
        job_line="echo I am here",
        post_job_lines=None,
        submit_cmd=None,
    ):
        """Select if run using SLURM script."""
        f = open(filename, "w")
        f.write("%s\n" % shell)
        f.write("#SBATCH --nodes=%d\n" % (nnodes))
        f.write("#SBATCH --ntasks-per-node=%d\n" % (cores))

        if walltime is not None:
            if isinstance(walltime, str):
                f.write("#SBATCH --time=%s\n" % walltime)
            else:
                ValueError("Provide walltime in a string format", walltime)

        if queue is not None:
            f.write("#SBATCH --partition=%s\n" % (queue))

        if account is not None:
            f.write("#SBATCH --account=%s\n" % (account))

        if memory is not None:
            f.write("#SBATCH --mem=%s\n" % (memory))

        f.write("#SBATCH --error=%s\n" % joberr)
        f.write("#SBATCH --output=%s\n" % jobout)
        if pre_job_lines is not None:
            f.write("%s\n" % pre_job_lines)
        if directory is not None:
            f.write("cd %s\n" % directory)
        if job_line is not None:
            f.write("%s\n" % job_line)
        if post_job_lines is not None:
            f.write("%s\n" % post_job_lines)
        f.close()

        if submit_cmd is not None:

            with open("jobid", "w") as f:
                p = subprocess.Popen(
                    submit_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
                )
                stdout, stderr = p.communicate()
                print("stdout,stderr", stdout, stderr)
                # job_id = str(stdout.split('Your job')[1].split(' ')[1])
                f.write(str(stdout))


"""
if __name__ == "__main__":
    Queue().pbs()
    Queue().slurm(filename="ll")
    Queue().head_node("echo ABC")
"""



"""Modules for generating input files for QE."""

import os
import requests
import tarfile
from jarvis.core.specie import Specie


class QEinfile(object):
    """
    Gnererate quantum espresso input files.

    jarvis.core.Atoms and Kpoints are required as inputs.
    For other input parameters,
    see https://www.quantum-espresso.org/Doc/INPUT_PW.html .
    """

    def __init__(
        self, atoms, kpoints, psp_dir=None, input_params={}, url=None
    ):
        """Initialize input parameters for qunatum espresso."""
        if input_params == {}:
            input_params = GenericInputs().geometry_optimization()
        if psp_dir is None:
            psp_dir = str(os.path.join(os.path.dirname(__file__), "QE_PSPs"))
            # Download GBRV PSPs by default
            if url is None:
                url = ("http://www.physics.rutgers.edu/"
                       "gbrv/all_pbesol_UPF_v1.5.tar.gz")
            if not os.path.exists(psp_dir):
                print("Downloading PSPs")
                tar_file_name = str(
                    os.path.join(os.path.dirname(__file__), "psp.tar.gz")
                )
                r = requests.get(url)
                f = open(tar_file_name, "wb")
                f.write(r.content)
                f.close()
                tar = tarfile.open(tar_file_name)
                tar.extractall(psp_dir)
                tar.close()
                os.remove(tar_file_name)

        self.psp_dir = psp_dir

        self.atoms = atoms
        input_params["system_params"]["nat"] = self.atoms.num_atoms
        self.species = self.atoms.uniq_species
        input_params["system_params"]["ntyp"] = len(self.species)

        self.kpoints = kpoints

        self.control_params = input_params["control_params"]
        self.control_params["pseudo_dir"] = str("'") + self.psp_dir + "'"
        self.system_params = input_params["system_params"]
        self.electron_params = input_params["electron_params"]
        self.ion_params = input_params["ion_params"]
        self.cell_params = input_params["cell_params"]
        self.input_params = input_params

    def dictionary_to_string(self, tags={}):
        """Convert a dictionary to string with '=' sign."""
        lines = ""
        for i, j in tags.items():
            if j is not None:
                lines = lines + str(i) + str(" = ") + str(j) + "\n"
        return lines

    def kpoints_to_string(self):
        """Convert a jarvis.core.Kpoints3D to string."""
        kp = ""
        kpoint_mode = self.kpoints._kpoint_mode
        if kpoint_mode == "automatic":
            kp = (
                kp
                + "K_POINTS automatic\n"
                + (" ".join(map(str, self.kpoints.kpts[0])) + " 0 0 0\n")
            )
        elif kpoint_mode == "linemode":
            points = ""
            for i in self.kpoints.kpts:
                points = points + " ".join(map(str, i)) + " 1.0" + "\n"
            kp = (
                kp
                + "K_POINTS crystal\n"
                + str(len(self.kpoints.kpts))
                + "\n"
                + points
            )

        else:
            print("Kpoint scheme not implemented except linemode, & automatic")
        return kp

    def get_psp(self, element):
        """Obtain psuedopotential for an element."""
        element = str(element).lower()
        for i in os.listdir(self.psp_dir):
            el = str(i.split("_")[0]).lower()
            if el == element:
                return i

    def atomic_species_string(self):
        """Obtain string for QE atomic species."""
        line = ""
        for i in self.species:
            line = line + (
                i
                + " "
                + str(round(Specie(i).atomic_mass, 2))
                + " "
                + str(self.get_psp(i))
                + "\n"
            )
        return line

    def atomic_pos(self):
        """Obtain string for QE atomic positions."""
        line = ""
        for i, j in zip(self.atoms.elements, self.atoms.frac_coords):
            line = line + str(i) + " " + " ".join(map(str, j)) + "\n"
        return line

    def atomic_cell_params(self):
        """Obtain string for QE atomic lattice parameters."""
        line = (
            str(self.atoms.lattice_mat[0][0])
            + " "
            + str(self.atoms.lattice_mat[0][1])
            + " "
            + str(self.atoms.lattice_mat[0][2])
            + "\n"
            + str(self.atoms.lattice_mat[1][0])
            + " "
            + str(self.atoms.lattice_mat[1][1])
            + " "
            + str(self.atoms.lattice_mat[1][2])
            + "\n"
            + str(self.atoms.lattice_mat[2][0])
            + " "
            + str(self.atoms.lattice_mat[2][1])
            + " "
            + str(self.atoms.lattice_mat[2][2])
        )
        return line

    def to_string(self):
        """Convert inputs to a string to write in file."""
        control = self.dictionary_to_string(self.control_params)
        system = self.dictionary_to_string(self.system_params)
        electrons = self.dictionary_to_string(self.electron_params)
        ions = self.dictionary_to_string(self.ion_params)
        cell = self.dictionary_to_string(self.cell_params)
        line = (
            "&control\n\n"
            + control
            + "/"
            + "\n&system\n\n"
            + system
            + "/"
            + "\n&electrons\n\n"
            + electrons
            + "/"
            + "\n&ions\n\n"
            + ions
            + "/"
            + "\n&cell\n\n"
            + cell
            + "/"
            + "\n"
            + "ATOMIC_SPECIES\n\n"
            + self.atomic_species_string()
            + "\n"
            + "ATOMIC_POSITIONS crystal\n\n"
            + self.atomic_pos()
            + "\n"
            + "CELL_PARAMETERS angstrom\n\n"
            + self.atomic_cell_params()
            + "\n\n"
            + self.kpoints_to_string()
        )
        return line

    def write_file(self, filename="qe.in"):
        """Write input file."""
        line = self.to_string()
        f = open(filename, "w")
        f.write(line)
        f.close()


class GenericInputs(object):
    """Obtain set of minimal QE input parameters."""

    def __init__(self):
        """Initialize with minimum parameters that can be updated."""
        self.sample_qe_inputs = {
            "control_params": {
                "calculation": "'scf'",
                "restart_mode": "'from_scratch'",
                "prefix": "'QE'",
                "outdir": "'./'",
                "tstress": ".true.",
                "tprnfor": ".true.",
                "disk_io": "'low'",
                "wf_collect": ".true.",
                "pseudo_dir": None,
                "verbosity": "'high'",
                "nstep": 100,
            },
            "system_params": {
                "ibrav": 0,
                "nat": None,
                "ntyp": None,
                "ecutwfc": 45,
                "ecutrho": 250,
                "q2sigma": 1,
                "ecfixed": 44.5,
                "qcutz": 800,
                "occupations": "'smearing'",
                "degauss": 0.01,
                "lda_plus_u": ".false.",
                "force_symmorphic": ".true.",
                "nosym": ".false.",
                "noinv": ".false.",
            },
            "electron_params": {
                "diagonalization": "'david'",
                "mixing_mode": "'local-TF'",
                "mixing_beta": 0.3,
                "conv_thr": "1d-9",
            },
            "ion_params": {"ion_dynamics": "'bfgs'"},
            "cell_params": {
                "cell_dynamics": "'bfgs'",
                "cell_dofree": "'all'",
                "cell_factor": 1.0,
            },
        }

    def geometry_optimization(self):
        """Obtain QE inputs for geometry optimization."""
        input = self.sample_qe_inputs
        input["control_params"]["calculation"] = "'vc-relax'"
        return input

    # def simple_scf(self):
    #    """Obtain QE inputs for single SCF."""
    #    input = self.sample_qe_inputs
    #    return input

    # def electron_band_structure(self):
    #    """Obtain QE inputs for bandstructure calculations optimization."""
    #    input = self.sample_qe_inputs
    #    # TODO: increase nbands
    #    input["control_params"]["calculation"] = "'nscf'"
    #    input["control_params"]["restart_mode"] = None
    #    return input


"""
if __name__ == "__main__":
    from jarvis.core.kpoints import Kpoints3D
    from jarvis.core.atoms import Atoms
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    print(Si)
    kp = Kpoints3D().automatic_length_mesh(
        lattice_mat=Si.lattice_mat, length=20
    )
    qe = QEinfile(Si, kp)
    qe.write_file()
    kp = Kpoints3D().kpath(atoms=Si)
    qe = QEinfile(Si, kp)
    qe.write_file("qe.in2")
    sp = qe.atomic_species_string()
    sp = qe.atomic_cell_params()
    print("sp", sp)
    print(qe.input_params['system_params']['nat'])
#  /users/knc6/Software/QE/q-e/bin/pw.x -i qe.in
"""



"""Module to calculate the dark-matter screening."""
import numpy as np


def peskin_schroeder_constant(bandgap=None, fermi_velocities=[], epsilon=[]):
    """
    Provide Peskin Scroeder constant for a material.

    Just proportionality constant. Doesn't take into account
    g and Lambda.

    Args:
       bandgap: benadgap in eV.

       fermi_velocities: vx, vy, vz

       epslion:  eigenvalues of epsilon tensor


    Return:
       1/(epsion*vF)
    """
    return 1 / (np.array(fermi_velocities) * np.array(epsilon))


"""
if __name__ == "__main__":
    epsilon = [187.5, 9.8, 90.9]
    fermi_velocities = [0.0029, 0.0050, 0.0021]
    print(
        peskin_schroeder_constant(
            bandgap=0.035, fermi_velocities=fermi_velocities, epsilon=epsilon
        )
    )
"""



"""
Modules for analyzing infrared intensities.

Please find more details in https://doi.org/10.1038/s41524-020-0337-2 .
"""
from jarvis.core.spectrum import Spectrum
import numpy as np


def normalize_vecs(phonon_eigenvectors, masses):
    """
    Return the eigenvectors after division of each component by sqrt(mass).

    Adapted from https://github.com/JMSkelton/Phonopy-Spectroscopy/
    """
    nmodes = len(phonon_eigenvectors)
    nmasses = len(masses)
    natoms = nmasses
    sqrt_masses = np.sqrt(masses)
    eigendisplacements = np.zeros((nmodes, natoms, 3), dtype=np.float64)
    for i in range(0, nmodes):
        eigenvector = phonon_eigenvectors[i]
        for j in range(0, natoms):
            eigendisplacements[i, j, :] = np.divide(
                eigenvector[j], sqrt_masses[j]
            )
    return eigendisplacements


def ir_intensity(
    phonon_eigenvectors=[],
    phonon_eigenvalues=[],
    masses=[],
    born_charges=[],
    smoothen=True,
):
    """Calculate IR intensity using DFPT."""
    eigendisplacements = normalize_vecs(phonon_eigenvectors, masses)
    becDim1, becDim2, becDim3 = np.shape(born_charges)
    freq = []
    ir_ints = []
    for i, v in zip(eigendisplacements, phonon_eigenvalues):
        eigendisplacement = i
        eigDim1, eigDim2 = np.shape(eigendisplacement)
        irIntensity = 0.0
        for a in range(0, 3):
            sumTemp1 = 0.0
            for j in range(0, eigDim1):
                sumTemp2 = 0.0
                for b in range(0, 3):
                    sumTemp2 += born_charges[j][a][b] * eigendisplacement[j][b]
                sumTemp1 += sumTemp2
            irIntensity += sumTemp1 ** 2
        freq.append(v * 33.35641)  # Thz to cm-1
        ir_ints.append(irIntensity)
    if smoothen:
        freq, ir_ints = Spectrum(x=freq, y=ir_ints).smoothen_spiky_spectrum()
    return freq, ir_ints


"""
if __name__ == "__main__":
    from jarvis.io.vasp.outputs import Vasprun, Outcar

    out = Outcar("../../tests/testfiles/io/vasp/OUTCAR.JVASP-39")
    vrun = Vasprun("../../tests/testfiles/io/vasp/vasprun.xml.JVASP-39")
    phonon_eigenvectors = vrun.dfpt_data["phonon_eigenvectors"]
    vrun_eigs = vrun.dfpt_data["phonon_eigenvalues"]
    phonon_eigenvalues = out.phonon_eigenvalues
    masses = vrun.dfpt_data["masses"]
    born_charges = vrun.dfpt_data["born_charges"]
    x, y = ir_intensity(
        phonon_eigenvectors=phonon_eigenvectors,
        phonon_eigenvalues=phonon_eigenvalues,
        masses=masses,
        born_charges=born_charges,
    )
    for i, j in zip(x, y):
        if j > 0.1:
            print(i, j)
    print()
    print(round(y[1], 2))
    # for i, j in zip(phonon_eigenvalues, vrun_eigs):
    #    print(i, j)
"""



from jarvis.analysis.darkmatter.metrics import peskin_schroeder_constant
import numpy as np

def test_peskin_schroeder():
    epsilon = [187.5, 9.8, 90.9]
    fermi_velocities = [0.0029, 0.0050, 0.0021]
    val = peskin_schroeder_constant(
        bandgap=0.035, fermi_velocities=fermi_velocities, epsilon=epsilon
    )
    assert round(val[0],5)==round(1.83908046,5)
    assert round(val[1],5)==round(20.40816327,5)
    assert round(val[2],5)==round(5.2386191,5)



"""Module for setting MAGMOM and AFM/FM orderings."""

from jarvis.analysis.structure.spacegroup import Spacegroup3D
import numpy as np
from collections import defaultdict
from jarvis.core.utils import check_match


class MagneticOrdering(object):
    """Provide modules for enumerating magnetic ordering analysis."""

    def __init__(self, atoms=None):
        """Initialize with Atoms."""
        self.atoms = atoms

    def apply_symmetry_operations(self, atoms, spg, tol=1e-4):
        """Figure out the effects of all the symmetry operations."""
        trans = spg._dataset["translations"]
        rots = spg._dataset["rotations"]
        coords = atoms.frac_coords % 1
        nat = atoms.num_atoms
        found_everything = True
        permutations = []
        for rot, tran in zip(rots, trans):
            order = np.zeros(nat, dtype=int)
            found = False
            for at in range(nat):
                pos_new = np.dot(coords[at, :], rot.transpose()) + tran
                pos_new = pos_new % 1
                for at2 in range(nat):
                    if check_match(pos_new, coords[at2, :], tol=tol):
                        found = True
                        order[at] = at2
            if not found:
                found_everything = False
            permutations.append(order)
        return permutations, found_everything

    def get_unique_magnetic_structures(
        self,
        atoms,
        supercell_dim=[1, 1, 1],
        magnetic_ions=None,
        noferri=True,
        magmom=3.0,
    ):
        """
        Generate supercells with unique magnetic orderings.

        noferri=False to get ferrimagnetic configurations.
        """
        if magnetic_ions is None:
            magnetic_ions = set(atoms.elements)

        ss = atoms.make_supercell(dim=supercell_dim)
        # spg = Spacegroup3D(atoms)
        spg = Spacegroup3D(atoms)  # kfg

        # Apply symmetry with various tolerances until we find one that works
        worked = False
        for tol in [1e-7, 1e-6, 1e-5, 1e-4, 1e-3, 1e-2, 1e-1]:
            permutations, worked = self.apply_symmetry_operations(
                ss, spg, tol=tol
            )
            if worked:
                print("applied sym ", tol)
                break
        if not worked:
            print("error in apply_symmetry_operations")

        print("number of sym permutations:", len(permutations))
        nat = ss.num_atoms

        magnetic_list = []
        magnetic_count = 0
        mag_dict = {}
        for i, el in enumerate(ss.elements):
            if el in magnetic_ions:
                magnetic_list.append(True)
                mag_dict[magnetic_count] = i
                magnetic_count += 1

            else:
                magnetic_list.append(False)

        print("magnetic count: ", magnetic_count)
        if magnetic_count == 0:
            print("no magnetic ions, what are you doing????")
            return

        # generate all magnetic configurations
        magnetic_list = []
        for i in range(2 ** (magnetic_count)):
            magnetic_list.append(np.zeros(magnetic_count))
            # magnetic_list[-1][0] = 5.0

        tmp = "00000000000000000000000000000000000000000000000000000000000000"
        if magnetic_count > 0:
            for i in range(2 ** (magnetic_count)):
                binary_int = bin(i).replace("0b", "")  # convert to binary
                total_int = tmp + binary_int

                for ii, d in enumerate(total_int[-magnetic_count:]):
                    if d == "0":
                        # print(i, ii, d)
                        magnetic_list[i][ii] = magmom
                    else:
                        # print(i, ii, d)
                        magnetic_list[i][ii] = -1 * magmom

        if (
            noferri
        ):  # get rid if ferrimagnetic configurations, only exact AFM and FM
            newlist = []
            for i in range(2 ** (magnetic_count)):
                if (
                    np.abs(
                        np.abs(np.sum(magnetic_list[i])) / abs(magmom)
                        - magnetic_count
                    )
                    < 1e-5
                    or np.abs(np.sum(magnetic_list[i])) < 1e-5
                ):
                    newlist.append(magnetic_list[i])
            magnetic_list = newlist

        # convert to all atoms in cell
        mag_all = []
        for mag in magnetic_list:
            z = np.zeros(nat)
            for i, m in enumerate(mag):
                z[mag_dict[i]] = m
            mag_all.append(z)

        print("generated, now apply sym opps to find unique")
        # apply symmetry ops
        symm_list = []
        for mag in mag_all:
            already_in_list = False
            for p in permutations:
                mag_new = mag[p]
                for s in symm_list:
                    if (
                        np.sum(np.abs(s - mag_new)) < 1e-5
                        or np.sum(np.abs(s + mag_new)) < 1e-5
                    ):  # either we found the same config, or same config * -1
                        already_in_list = True
                        break
            if not already_in_list:  # then we need it
                symm_list.append(mag)

        print("number of unique configs: ", len(symm_list))
        return symm_list, ss

    def tc_mean_field(self, energies=[-2, -1]):
        """Curie temperature using mean-field theory."""
        info = {}
        mag_atoms = self.get_mag_ions()
        deltaE = max(energies) - min(energies)
        print(deltaE)
        kB = 8.617333262e-05
        elements_dict = defaultdict(int)
        for i in self.atoms.elements:
            if i in mag_atoms:
                elements_dict[i] += 1
        n_mag_elements = sum(elements_dict.values())
        Tc = deltaE / (3 * kB) / n_mag_elements
        # Tc = 2 * deltaE / (3 * kB) / n_mag_elements
        info["Tc"] = Tc
        info["deltaE"] = deltaE
        info["n_mag_elements"] = n_mag_elements
        return info

    def get_mag_ions(self):
        """List all magnetic atoms in the Atoms object."""
        all_mag_elements = [
            "Ti",
            "V",
            "Cr",
            "Mn",
            "Fe",
            "Co",
            "Ni",
            "Cu",
            "Ru",
            "Ir",
            "Rh",
            "Os",
            "Rb",
            "Sc",
        ]
        els = self.atoms.elements
        mag_ions = list(set(all_mag_elements).intersection(set(els)))
        return mag_ions

    def get_minimum_configs(self, min_configs=3, enforce_primitive=True):
        """Get minimum number of spin structures for Tc calculations."""
        atoms = self.atoms
        if enforce_primitive:
            atoms = atoms.get_primitive_atoms
        lengths = atoms.lattice.lat_lengths()
        index_to_expand = np.argsort(lengths)
        dim = np.array([1, 1, 1])
        mag_ions = self.get_mag_ions()
        symm_list, ss = self.get_unique_magnetic_structures(
            atoms, supercell_dim=dim, magnetic_ions=mag_ions, noferri=True
        )
        # kfg
        if len(symm_list) < min_configs:
            symm_list, ss = self.get_unique_magnetic_structures(
                atoms, supercell_dim=dim, magnetic_ions=mag_ions, noferri=True
            )

        count = 0
        while len(symm_list) < min_configs:
            dim[index_to_expand[count]] += 1
            symm_list, ss = self.get_unique_magnetic_structures(
                atoms,
                supercell_dim=dim,
                magnetic_ions=self.get_mag_ions(),
                noferri=True,
            )
            # kfg
            if len(symm_list) < min_configs:
                symm_list, ss = self.get_unique_magnetic_structures(
                    atoms,
                    supercell_dim=dim,
                    magnetic_ions=self.get_mag_ions(),
                    noferri=False,
                )

            count = count + 1
            if count > 2:
                count = 0
            print("Supercell dimension", dim)
        return symm_list, ss



"""Module for post-processing phonopy outputs."""
import yaml
import matplotlib.pyplot as plt
from yaml import Loader
import numpy as np
from jarvis.io.phonopy.inputs import PhonopyInputs

# from jarvis.analysis.structure.spacegroup import Spacegroup3D
# from jarvis.io.wannier.outputs import WannierHam
from jarvis.io.phonopy.fcmat2hr import get_phonon_hr

try:
    from phonopy import Phonopy
    from phonopy.units import VaspToTHz
    from phonopy.structure.cells import determinant
    from phonopy.structure.cells import get_reduced_bases
except Exception as exp:
    print("Phonopy is not installed.", exp)
    pass


def bandstructure_plot(band_yaml="", plot=False):
    """Get phonopy bandstructure info."""
    with open(band_yaml, "r") as f:
        data = yaml.load(f, Loader=Loader)
    frequencies = []
    distances = []
    qpoints = []
    label_points = []
    labels = []
    for j, v in enumerate(data["phonon"]):
        if "label" in v and v["label"] != "None":
            labels.append(v["label"])
            label_points.append(v["distance"])
        frequencies.append([f["frequency"] for f in v["band"]])
        qpoints.append(v["q-position"])
        distances.append(v["distance"])
    if plot:
        for i in range(np.array(frequencies).shape[1]):
            plt.plot(distances, np.array(frequencies)[:, i])
        plt.xticks(label_points, labels)
    return frequencies, distances, labels, label_points


def total_dos(tot_dos="", plot=False):
    """Get total dos info."""
    f = open(tot_dos, "r")
    freq = []
    pdos = []
    for lines in f.readlines():
        if not str(lines.split()[0]).startswith("#"):
            #   print (lines)
            # else:
            freq.append(float(lines.split()[0]))
            pdos.append(float(lines.split()[1]))
    if plot:
        plt.plot(freq, pdos)
    return freq, pdos


def read_fc(filename="FORCE_CONSTANTS"):
    """Read phonopy generated force constants."""
    f = open(filename, "r")
    lines = f.read().splitlines()
    f.close()
    natoms = int(lines[0].split()[0])
    fc = np.zeros((natoms, natoms, 3, 3), dtype="double")
    # print ('natoms=',natoms)
    for ii, i in enumerate(lines):
        if ii > 0 and ii % 4 == 0:
            atoms_ids = [int(a) for a in lines[ii - 3].split()]
            vals = (
                str(lines[ii - 2])
                + " "
                + str(lines[ii - 1])
                + " "
                + (lines[ii])
            )
            vals = np.array(vals.split(), dtype="double").reshape(3, 3)
            fc[atoms_ids[0] - 1, atoms_ids[1] - 1] = vals
    return fc


def get_phonon_tb(
    # phonopy_atoms=[],
    atoms=[],
    fc=[],
    out_file="phonopyTB_hr.dat",
    distance_to_A=1.0,
    scell=np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]]),
    factor=VaspToTHz,
    symprec=1e-05,
    displacement_distance=0.01,
):
    """Generate phonon TB Hamiltonia, along with WannierHamn."""
    # Forked from Wannier-tools
    unitcell = atoms.phonopy_converter()
    # unitcell = phonopy_atoms
    prim_mat = np.array(
        PhonopyInputs(atoms).prim_axis().split("=")[1].split(), dtype="float"
    ).reshape(3, 3)
    # print("cell", unitcell.cell)
    num_atom = unitcell.get_number_of_atoms()
    num_satom = determinant(scell) * num_atom
    if fc.shape[0] != num_satom:
        print("Check Force constant matrix.")
    phonon = Phonopy(
        unitcell,
        scell,
        primitive_matrix=prim_mat,
        factor=factor,
        dynamical_matrix_decimals=None,
        force_constants_decimals=None,
        symprec=symprec,
        is_symmetry=True,
        use_lapack_solver=False,
        log_level=1,
    )

    supercell = phonon.get_supercell()
    primitive = phonon.get_primitive()
    # Set force constants
    phonon.set_force_constants(fc)
    phonon._set_dynamical_matrix()
    dmat = phonon._dynamical_matrix
    # rescale fcmat by THZ**2
    fcmat = dmat._force_constants * factor ** 2  # FORCE_CONSTANTS
    # fcmat = dmat._force_constants * factor ** 2  # FORCE_CONSTANTS
    smallest_vectors = dmat._smallest_vectors
    # mass = dmat._mass
    mass = dmat._pcell.get_masses()
    print("mass=", mass)
    multi = dmat._multiplicity
    reduced_bases = get_reduced_bases(supercell.get_cell(), symprec)
    positions = np.dot(supercell.get_positions(), np.linalg.inv(reduced_bases))
    # for pos in positions: pos -= np.rint(pos)
    relative_scale = np.dot(reduced_bases, np.linalg.inv(primitive.get_cell()))
    super_pos = np.zeros((num_satom, 3), dtype=np.float64)
    for i in range(num_satom):
        super_pos[i] = np.dot(positions[i], relative_scale)
    p2s_map = dmat._p2s_map = primitive.get_primitive_to_supercell_map()
    s2p_map = dmat._s2p_map = primitive.get_supercell_to_primitive_map()
    num_satom = supercell.get_number_of_atoms()
    num_patom = primitive.get_number_of_atoms()
    get_phonon_hr(
        fcmat,
        smallest_vectors,
        mass,
        multi,
        super_pos,
        p2s_map,
        s2p_map,
        num_satom,
        num_patom,
        out_file,
    )
    print("phonopy_TB.dat generated! ")


"""
if __name__ == "__main__":
    from phonopy.interface.vasp import read_vasp
    from jarvis.core.atoms import Atoms
    pos = "POSCAR"
    fc_file = "FORCE_CONSTANTS"
    a = Atoms.from_poscar(pos)
    fc = read_fc(fc_file)
    phonopy_atoms = read_vasp(pos)
    get_phonon_tb(phonopy_atoms=phonopy_atoms, fc=fc, atoms=a)
    cvn = Spacegroup3D(a).conventional_standard_structure
    w = WannierHam("phonopyTB_hr.dat")
    w.get_bandstructure_plot(atoms=cvn, yrange=[0, 550])
"""



"""Module to run BoltzTrap. Assumes x_trans in PATH."""
import os
from jarvis.io.boltztrap.inputs import WriteInputs


def run_boltztrap(
    path="MAIN-RELAX", cmd="~/anaconda2/bin/x_trans BoltzTraP -so"
):
    """Run boltztrap program with vasprun.xml."""
    os.chdir(path)
    try:
        vrun = os.path.join(path, "vasprun.xml")
        inp = WriteInputs(vasprun_path=vrun)
        inp.write_energy()
        inp.write_struct()
        inp.write_intrans()
        os.system(cmd)
    except Exception:
        print("Error running boltztrap, check x_trans and vasprun.")
        pass
    os.chdir(path)
